@@ -61,8 +61,7 @@ def get_acl(headers, body, bucket_owner, object_owner=None):
     if acl is None:
         # Get acl from request body if possible.
         if not body:
-            msg = 'Your request was missing a required header'
-            raise MissingSecurityHeader(msg, missing_header_name='x-amz-acl')
+            raise MissingSecurityHeader(missing_header_name='x-amz-acl')
         try:
             elem = fromstring(body, ACL.root_tag)
             acl = ACL.from_elem(elem)
@@ -85,7 +84,7 @@ def get_acl_handler(controller_name):
         # pylint: disable-msg=E1101
         for handler in base_klass.__subclasses__():
             handler_suffix_len = len('AclHandler') \
-                if not handler.__name__ == 'S3AclHandler' else len('Hanlder')
+                if not handler.__name__ == 'S3AclHandler' else len('Handler')
             if handler.__name__[:-handler_suffix_len] == controller_name:
                 return handler
     return BaseAclHandler
