@@ -59,7 +59,8 @@ from swift3.response import InvalidArgument, ErrorResponse, MalformedXML, \
     InvalidRequest, HTTPOk, HTTPNoContent, NoSuchKey, NoSuchUpload, \
     NoSuchBucket
 from swift3.exception import BadSwiftRequest
-from swift3.utils import LOGGER, unique_id, MULTIUPLOAD_SUFFIX, S3Timestamp
+from swift3.utils import LOGGER, unique_id, MULTIUPLOAD_SUFFIX, S3Timestamp, \
+    sysmeta_header
 from swift3.etree import Element, SubElement, fromstring, tostring, \
     XMLSyntaxError, DocumentInvalid
 from swift3.cfg import CONF
@@ -333,6 +334,16 @@ class UploadsController(Controller):
         upload_id = unique_id()
 
         container = req.container_name + MULTIUPLOAD_SUFFIX
+
+        content_type = req.headers.get('Content-Type')
+        if content_type:
+            req.headers[sysmeta_header('object', 'has-content-type')] = 'yes'
+            req.headers[
+                sysmeta_header('object', 'content-type')] = content_type
+        else:
+            req.headers[sysmeta_header('object', 'has-content-type')] = 'no'
+        req.headers['Content-Type'] = 'application/directory'
+
         try:
             req.get_response(self.app, 'PUT', container, '')
         except BucketAlreadyExists:
@@ -340,6 +351,9 @@ class UploadsController(Controller):
 
         obj = '%s/%s' % (req.object_name, upload_id)
 
+        req.headers.pop('Etag', None)
+        req.headers.pop('Content-Md5', None)
+
         req.get_response(self.app, 'PUT', container, obj, body='')
 
         result_elem = Element('InitiateMultipartUploadResult')
@@ -508,8 +522,22 @@ class UploadController(Controller):
             _key = key.lower()
             if _key.startswith('x-amz-meta-'):
                 headers['x-object-meta-' + _key[11:]] = val
-            elif _key == 'content-type':
-                headers['Content-Type'] = val
+
+        hct_header = sysmeta_header('object', 'has-content-type')
+        if resp.sysmeta_headers.get(hct_header) == 'yes':
+            content_type = resp.sysmeta_headers.get(
+                sysmeta_header('object', 'content-type'))
+        elif hct_header in resp.sysmeta_headers:
+            # has-content-type is present but false, so no content type was
+            # set on initial upload. In that case, we won't set one on our
+            # PUT request. Swift will end up guessing one based on the
+            # object name.
+            content_type = None
+        else:
+            content_type = resp.headers.get('Content-Type')
+
+        if content_type:
+            headers['Content-Type'] = content_type
 
         # Query for the objects in the segments area to make sure it completed
         query = {
@@ -530,6 +558,9 @@ class UploadController(Controller):
         previous_number = 0
         try:
             xml = req.xml(MAX_COMPLETE_UPLOAD_BODY_SIZE)
+            if not xml:
+                raise InvalidRequest(msg='You must specify at least one part')
+
             complete_elem = fromstring(xml, 'CompleteMultipartUpload')
             for part_elem in complete_elem.iterchildren('Part'):
                 part_number = int(part_elem.find('./PartNumber').text)
@@ -549,6 +580,7 @@ class UploadController(Controller):
                     raise InvalidPart(upload_id=upload_id,
                                       part_number=part_number)
 
+                info['size_bytes'] = int(info['size_bytes'])
                 manifest.append(info)
         except (XMLSyntaxError, DocumentInvalid):
             raise MalformedXML()
@@ -561,15 +593,20 @@ class UploadController(Controller):
 
         # Following swift commit 7f636a5, zero-byte segments aren't allowed,
         # even as the final segment
-        if int(info['size_bytes']) == 0:
-            manifest.pop()
+        empty_seg = None
+        if manifest[-1]['size_bytes'] == 0:
+            empty_seg = manifest.pop()
 
-            # Ordinarily, we just let SLO check segment sizes. However, we
-            # just popped off a zero-byte segment; if there was a second
-            # zero-byte segment and it was at the end, it would succeed on
-            # Swift < 2.6.0 and fail on newer Swift. It seems reasonable that
-            # it should always fail.
-            if manifest and int(manifest[-1]['size_bytes']) == 0:
+            # We'll check the sizes of all except the last segment below, but
+            # since we just popped off a zero-byte segment, we should check
+            # that last segment, too.
+            if manifest and manifest[-1]['size_bytes'] < CONF.min_segment_size:
+                raise EntityTooSmall()
+
+        # Check the size of each segment except the last and make sure they are
+        # all more than the minimum upload chunk size
+        for info in manifest[:-1]:
+            if info['size_bytes'] < CONF.min_segment_size:
                 raise EntityTooSmall()
 
         try:
@@ -586,14 +623,8 @@ class UploadController(Controller):
                                         headers=headers)
         except BadSwiftRequest as e:
             msg = str(e)
-            msg_pre_260 = 'Each segment, except the last, must be at least '
-            # see https://github.com/openstack/swift/commit/c0866ce
-            msg_260 = ('too small; each segment, except the last, must be '
-                       'at least ')
-            # see https://github.com/openstack/swift/commit/7f636a5
-            msg_post_260 = 'too small; each segment must be at least 1 byte'
-            if msg.startswith(msg_pre_260) or \
-                    msg_260 in msg or msg_post_260 in msg:
+            expected_msg = 'too small; each segment must be at least 1 byte'
+            if expected_msg in msg:
                 # FIXME: AWS S3 allows a smaller object than 5 MB if there is
                 # only one part.  Use a COPY request to copy the part object
                 # from the segments container instead.
@@ -601,15 +632,18 @@ class UploadController(Controller):
             else:
                 raise
 
-        if int(info['size_bytes']) == 0:
+        if empty_seg:
             # clean up the zero-byte segment
-            empty_seg_cont, empty_seg_name = info['path'].split('/', 2)[1:]
+            _, empty_seg_cont, empty_seg_name = empty_seg['path'].split('/', 2)
             req.get_response(self.app, 'DELETE',
                              container=empty_seg_cont, obj=empty_seg_name)
 
         # clean up the multipart-upload record
         obj = '%s/%s' % (req.object_name, upload_id)
-        req.get_response(self.app, 'DELETE', container, obj)
+        try:
+            req.get_response(self.app, 'DELETE', container, obj)
+        except NoSuchKey:
+            pass  # We know that this existed long enough for us to HEAD
 
         result_elem = Element('CompleteMultipartUploadResult')
 
