@@ -14,12 +14,15 @@
 # limitations under the License.
 
 import base64
+from collections import defaultdict
 from email.header import Header
-from hashlib import sha256, md5
+from hashlib import sha1, sha256, md5
+import hmac
 import re
 import six
+# pylint: disable-msg=import-error
+from six.moves.urllib.parse import quote, unquote, parse_qsl
 import string
-from urllib import quote, unquote
 
 from swift.common.utils import split_path
 from swift.common import swob
@@ -48,7 +51,7 @@ from swift3.response import AccessDenied, InvalidArgument, InvalidDigest, \
     BadDigest, AuthorizationHeaderMalformed, AuthorizationQueryParametersError
 from swift3.exception import NotS3Request, BadSwiftRequest
 from swift3.utils import utf8encode, LOGGER, check_path_header, S3Timestamp, \
-    mktime
+    mktime, MULTIUPLOAD_SUFFIX
 from swift3.cfg import CONF
 from swift3.subresource import decode_acl, encode_acl
 from swift3.utils import sysmeta_header, validate_bucket_name
@@ -71,6 +74,20 @@ ALLOWED_SUB_RESOURCES = sorted([
 MAX_32BIT_INT = 2147483647
 SIGV2_TIMESTAMP_FORMAT = '%Y-%m-%dT%H:%M:%S'
 SIGV4_X_AMZ_DATE_FORMAT = '%Y%m%dT%H%M%SZ'
+SERVICE = 's3'  # useful for mocking out in tests
+
+
+def _header_strip(value):
+    # S3 seems to strip *all* control characters
+    if value is None:
+        return None
+    stripped = _header_strip.re.sub('', value)
+    if value and not stripped:
+        # If there's nothing left after stripping,
+        # behave as though it wasn't provided
+        return None
+    return stripped
+_header_strip.re = re.compile('^[\x00-\x20]*|[\x00-\x20]*$')
 
 
 def _header_acl_property(resource):
@@ -96,6 +113,16 @@ class SigV4Mixin(object):
     A request class mixin to provide S3 signature v4 functionality
     """
 
+    def check_signature(self, secret):
+        user_signature = self.signature
+        derived_secret = 'AWS4' + secret
+        for scope_piece in self.scope:
+            derived_secret = hmac.new(
+                derived_secret, scope_piece, sha256).digest()
+        valid_signature = hmac.new(
+            derived_secret, self.string_to_sign, sha256).hexdigest()
+        return user_signature == valid_signature
+
     @property
     def _is_query_auth(self):
         return 'X-Amz-Credential' in self.params
@@ -125,10 +152,15 @@ class SigV4Mixin(object):
                 raise AccessDenied('AWS authentication requires a valid Date '
                                    'or x-amz-date header')
 
+            if timestamp < 0:
+                raise AccessDenied('AWS authentication requires a valid Date '
+                                   'or x-amz-date header')
+
             try:
                 self._timestamp = S3Timestamp(timestamp)
             except ValueError:
-                raise AccessDenied()
+                # Must be far-future; blame clock skew
+                raise RequestTimeTooSkewed()
 
         return self._timestamp
 
@@ -235,9 +267,18 @@ class SigV4Mixin(object):
 
         :return : dict of headers to sign, the keys are all lower case
         """
-        headers_lower_dict = dict(
-            (k.lower().strip(), ' '.join((v or '').strip().split()))
-            for (k, v) in six.iteritems(self.headers))
+        if 'headers_raw' in self.environ:  # eventlet >= 0.19.0
+            # See https://github.com/eventlet/eventlet/commit/67ec999
+            headers_lower_dict = defaultdict(list)
+            for key, value in self.environ['headers_raw']:
+                headers_lower_dict[key.lower().strip()].append(
+                    ' '.join(_header_strip(value or '').split()))
+            headers_lower_dict = {k: ','.join(v)
+                                  for k, v in headers_lower_dict.items()}
+        else:  # mostly-functional fallback
+            headers_lower_dict = dict(
+                (k.lower().strip(), ' '.join(_header_strip(v or '').split()))
+                for (k, v) in six.iteritems(self.headers))
 
         if 'host' in headers_lower_dict and re.match(
                 'Boto/2.[0-9].[0-2]',
@@ -249,7 +290,7 @@ class SigV4Mixin(object):
                 headers_lower_dict['host'].split(':')[0]
 
         headers_to_sign = [
-            (key, value) for key, value in headers_lower_dict.items()
+            (key, value) for key, value in sorted(headers_lower_dict.items())
             if key in self._signed_headers]
 
         if len(headers_to_sign) != len(self._signed_headers):
@@ -260,7 +301,7 @@ class SigV4Mixin(object):
             # process.
             raise SignatureDoesNotMatch()
 
-        return dict(headers_to_sign)
+        return headers_to_sign
 
     def _canonical_uri(self):
         """
@@ -268,13 +309,7 @@ class SigV4Mixin(object):
         """
         return self.environ.get('RAW_PATH_INFO', self.path)
 
-    def _string_to_sign(self):
-        """
-        Create 'StringToSign' value in Amazon terminology for v4.
-        """
-        scope = (self.timestamp.amz_date_format.split('T')[0] +
-                 '/' + CONF.location + '/s3/aws4_request')
-
+    def _canonical_request(self):
         # prepare 'canonical_request'
         # Example requests are like following:
         #
@@ -304,13 +339,12 @@ class SigV4Mixin(object):
         # host:iam.amazonaws.com
         # x-amz-date:20150830T123600Z
         headers_to_sign = self._headers_to_sign()
-        cr.append('\n'.join(
-            ['%s:%s' % (key, value) for key, value in
-             sorted(headers_to_sign.items())]) + '\n')
+        cr.append(''.join('%s:%s\n' % (key, value)
+                          for key, value in headers_to_sign))
 
         # 5. Add signed headers into canonical request like
         # content-type;host;x-amz-date
-        cr.append(';'.join(sorted(headers_to_sign)))
+        cr.append(';'.join(k for k, v in headers_to_sign))
 
         # 6. Add payload string at the tail
         if 'X-Amz-Credential' in self.params:
@@ -323,12 +357,21 @@ class SigV4Mixin(object):
         else:
             hashed_payload = self.headers['X-Amz-Content-SHA256']
         cr.append(hashed_payload)
-        canonical_request = '\n'.join(cr)
+        return '\n'.join(cr).encode('utf-8')
 
-        return ('AWS4-HMAC-SHA256' + '\n'
-                + self.timestamp.amz_date_format + '\n'
-                + scope + '\n'
-                + sha256(canonical_request.encode('utf-8')).hexdigest())
+    @property
+    def scope(self):
+        return [self.timestamp.amz_date_format.split('T')[0],
+                CONF.location, SERVICE, 'aws4_request']
+
+    def _string_to_sign(self):
+        """
+        Create 'StringToSign' value in Amazon terminology for v4.
+        """
+        return '\n'.join(['AWS4-HMAC-SHA256',
+                          self.timestamp.amz_date_format,
+                          '/'.join(self.scope),
+                          sha256(self._canonical_request()).hexdigest()])
 
 
 def get_request_class(env):
@@ -363,11 +406,19 @@ class Request(swob.Request):
         # NOTE: app is not used by this class, need for compatibility of S3acl
         swob.Request.__init__(self, env)
         self._timestamp = None
-        self.access_key, signature = self._parse_auth_info()
+        self.access_key, self.signature = self._parse_auth_info()
         self.bucket_in_host = self._parse_host()
         self.container_name, self.object_name = self._parse_uri()
         self._validate_headers()
-        self.token = base64.urlsafe_b64encode(self._string_to_sign())
+        # Lock in string-to-sign now, before we start messing with query params
+        self.string_to_sign = self._string_to_sign()
+        self.environ['swift3.auth_details'] = {
+            'access_key': self.access_key,
+            'signature': self.signature,
+            'string_to_sign': self.string_to_sign,
+            'check_signature': self.check_signature,
+        }
+        self.token = None
         self.account = None
         self.user_id = None
         self.slo_enabled = slo_enabled
@@ -380,11 +431,17 @@ class Request(swob.Request):
         # b626a3ca86e467fc7564eac236b9ee2efd49bdcc, the s3token is in swift3
         # repo so probably we need to change s3token to support v4 format.
         self.headers['Authorization'] = 'AWS %s:%s' % (
-            self.access_key, signature)
+            self.access_key, self.signature)
         # Avoids that swift.swob.Response replaces Location header value
         # by full URL when absolute path given. See swift.swob for more detail.
         self.environ['swift.leave_relative_location'] = True
 
+    def check_signature(self, secret):
+        user_signature = self.signature
+        valid_signature = base64.b64encode(hmac.new(
+            secret, self.string_to_sign, sha1).digest()).strip()
+        return user_signature == valid_signature
+
     @property
     def timestamp(self):
         """
@@ -408,10 +465,14 @@ class Request(swob.Request):
                 raise AccessDenied('AWS authentication requires a valid Date '
                                    'or x-amz-date header')
 
+            if timestamp < 0:
+                raise AccessDenied('AWS authentication requires a valid Date '
+                                   'or x-amz-date header')
             try:
                 self._timestamp = S3Timestamp(timestamp)
             except ValueError:
-                raise AccessDenied()
+                # Must be far-future; blame clock skew
+                raise RequestTimeTooSkewed()
 
         return self._timestamp
 
@@ -571,8 +632,8 @@ class Request(swob.Request):
 
         self._validate_dates()
 
-        if 'Content-MD5' in self.headers:
-            value = self.headers['Content-MD5']
+        value = _header_strip(self.headers.get('Content-MD5'))
+        if value is not None:
             if not re.match('^[A-Za-z0-9+/]+={0,2}$', value):
                 # Non-base64-alphabet characters in value.
                 raise InvalidDigest(content_md5=value)
@@ -630,7 +691,7 @@ class Request(swob.Request):
         """
         raise AttributeError("No attribute 'body'")
 
-    def xml(self, max_length, check_md5=False):
+    def xml(self, max_length):
         """
         Similar to swob.Request.body, but it checks the content length before
         creating a body string.
@@ -645,11 +706,13 @@ class Request(swob.Request):
         if self.message_length() > max_length:
             raise MalformedXML()
 
-        # Limit the read similar to how SLO handles manifests
-        body = self.body_file.read(max_length)
-
-        if check_md5:
-            self.check_md5(body)
+        if te or self.message_length():
+            # Limit the read similar to how SLO handles manifests
+            body = self.body_file.read(max_length)
+        else:
+            # No (or zero) Content-Length provided, and not chunked transfer;
+            # no body. Assume zero-length, and enforce a required body below.
+            return None
 
         return body
 
@@ -677,13 +740,30 @@ class Request(swob.Request):
 
         :returns: the source HEAD response
         """
-        if 'X-Amz-Copy-Source' not in self.headers:
+        try:
+            src_path = self.headers['X-Amz-Copy-Source']
+        except KeyError:
             return None
 
-        src_path = unquote(self.headers['X-Amz-Copy-Source'])
-        src_path = src_path if src_path.startswith('/') else \
-            ('/' + src_path)
+        if '?' in src_path:
+            src_path, qs = src_path.split('?', 1)
+            query = parse_qsl(qs, True)
+            if not query:
+                pass  # ignore it
+            elif len(query) > 1 or query[0][0] != 'versionId':
+                raise InvalidArgument('X-Amz-Copy-Source',
+                                      self.headers['X-Amz-Copy-Source'],
+                                      'Unsupported copy source parameter.')
+            elif query[0][1] != 'null':
+                # TODO: once we support versioning, we'll need to translate
+                # src_path to the proper location in the versions container
+                raise S3NotImplemented('Versioning is not yet supported')
+            self.headers['X-Amz-Copy-Source'] = src_path
+
+        src_path = unquote(src_path)
+        src_path = src_path if src_path.startswith('/') else ('/' + src_path)
         src_bucket, src_obj = split_path(src_path, 0, 2, True)
+
         headers = swob.HeaderKeyDict()
         headers.update(self._copy_source_headers())
 
@@ -725,42 +805,54 @@ class Request(swob.Request):
         """
         amz_headers = {}
 
-        buf = "%s\n%s\n%s\n" % (self.method,
-                                self.headers.get('Content-MD5', ''),
-                                self.headers.get('Content-Type') or '')
+        buf = [self.method,
+               _header_strip(self.headers.get('Content-MD5')) or '',
+               _header_strip(self.headers.get('Content-Type')) or '']
 
-        for amz_header in sorted((key.lower() for key in self.headers
-                                  if key.lower().startswith('x-amz-'))):
-            amz_headers[amz_header] = self.headers[amz_header]
+        if 'headers_raw' in self.environ:  # eventlet >= 0.19.0
+            # See https://github.com/eventlet/eventlet/commit/67ec999
+            amz_headers = defaultdict(list)
+            for key, value in self.environ['headers_raw']:
+                key = key.lower()
+                if not key.startswith('x-amz-'):
+                    continue
+                amz_headers[key.strip()].append(value.strip())
+            amz_headers = dict((key, ','.join(value))
+                               for key, value in amz_headers.items())
+        else:  # mostly-functional fallback
+            amz_headers = dict((key.lower(), value)
+                               for key, value in self.headers.items()
+                               if key.lower().startswith('x-amz-'))
 
         if self._is_header_auth:
             if 'x-amz-date' in amz_headers:
-                buf += "\n"
+                buf.append('')
             elif 'Date' in self.headers:
-                buf += "%s\n" % self.headers['Date']
+                buf.append(self.headers['Date'])
         elif self._is_query_auth:
-            buf += "%s\n" % self.params['Expires']
+            buf.append(self.params['Expires'])
         else:
             # Should have already raised NotS3Request in _parse_auth_info,
             # but as a sanity check...
             raise AccessDenied()
 
-        for k in sorted(key.lower() for key in amz_headers):
-            buf += "%s:%s\n" % (k, amz_headers[k])
+        for key, value in sorted(amz_headers.items()):
+            buf.append("%s:%s" % (key, value))
 
         path = self._canonical_uri()
         if self.query_string:
             path += '?' + self.query_string
+        params = []
         if '?' in path:
             path, args = path.split('?', 1)
-            params = []
             for key, value in sorted(self.params.items()):
                 if key in ALLOWED_SUB_RESOURCES:
                     params.append('%s=%s' % (key, value) if value else key)
-            if params:
-                return '%s%s?%s' % (buf, path, '&'.join(params))
-
-        return buf + path
+        if params:
+            buf.append('%s?%s' % (path, '&'.join(params)))
+        else:
+            buf.append(path)
+        return '\n'.join(buf)
 
     @property
     def controller_name(self):
@@ -831,15 +923,48 @@ class Request(swob.Request):
 
         env = self.environ.copy()
 
-        for key in self.environ:
-            if key.startswith('HTTP_X_AMZ_META_'):
-                if not(set(env[key]).issubset(string.printable)):
-                    env[key] = Header(env[key], 'UTF-8').encode()
-                    if env[key].startswith('=?utf-8?q?'):
-                        env[key] = '=?UTF-8?Q?' + env[key][10:]
-                    elif env[key].startswith('=?utf-8?b?'):
-                        env[key] = '=?UTF-8?B?' + env[key][10:]
-                env['HTTP_X_OBJECT_META_' + key[16:]] = env[key]
+        def sanitize(value):
+            if set(value).issubset(string.printable):
+                return value
+
+            value = Header(value, 'UTF-8').encode()
+            if value.startswith('=?utf-8?q?'):
+                return '=?UTF-8?Q?' + value[10:]
+            elif value.startswith('=?utf-8?b?'):
+                return '=?UTF-8?B?' + value[10:]
+            else:
+                return value
+
+        if 'headers_raw' in env:  # eventlet >= 0.19.0
+            # See https://github.com/eventlet/eventlet/commit/67ec999
+            for key, value in env['headers_raw']:
+                if not key.lower().startswith('x-amz-meta-'):
+                    continue
+                # AWS ignores user-defined headers with these characters
+                if any(c in key for c in ' "),/;<=>?@[\\]{}'):
+                    # NB: apparently, '(' *is* allowed
+                    continue
+                # Note that this may have already been deleted, e.g. if the
+                # client sent multiple headers with the same name, or both
+                # x-amz-meta-foo-bar and x-amz-meta-foo_bar
+                env.pop('HTTP_' + key.replace('-', '_').upper(), None)
+                # Need to preserve underscores. Since we know '=' can't be
+                # present, quoted-printable seems appropriate.
+                key = key.replace('_', '=5F').replace('-', '_').upper()
+                key = 'HTTP_X_OBJECT_META_' + key[11:]
+                if key in env:
+                    env[key] += ',' + sanitize(value)
+                else:
+                    env[key] = sanitize(value)
+        else:  # mostly-functional fallback
+            for key in self.environ:
+                if not key.startswith('HTTP_X_AMZ_META_'):
+                    continue
+                # AWS ignores user-defined headers with these characters
+                if any(c in key for c in ' "),/;<=>?@[\\]{}'):
+                    # NB: apparently, '(' *is* allowed
+                    continue
+                env['HTTP_X_OBJECT_META_' + key[16:]] = sanitize(env[key])
                 del env[key]
 
         if 'HTTP_X_AMZ_COPY_SOURCE' in env:
@@ -935,7 +1060,7 @@ class Request(swob.Request):
 
         return code_map[method]
 
-    def _swift_error_codes(self, method, container, obj):
+    def _swift_error_codes(self, method, container, obj, env, app):
         """
         Returns a dict from expected Swift error codes to the corresponding S3
         error responses.
@@ -968,13 +1093,25 @@ class Request(swob.Request):
             }
         else:
             # Swift object access.
+
+            # 404s differ depending upon whether the bucket exists
+            # Note that base-container-existence checks happen elsewhere for
+            # multi-part uploads, and get_container_info should be pulling
+            # from the env cache
+            def not_found_handler():
+                if container.endswith(MULTIUPLOAD_SUFFIX) or \
+                        is_success(get_container_info(
+                            env, app, swift_source='S3').get('status')):
+                    return NoSuchKey(obj)
+                return NoSuchBucket(container)
+
             code_map = {
                 'HEAD': {
-                    HTTP_NOT_FOUND: (NoSuchKey, obj),
+                    HTTP_NOT_FOUND: not_found_handler,
                     HTTP_PRECONDITION_FAILED: PreconditionFailed,
                 },
                 'GET': {
-                    HTTP_NOT_FOUND: (NoSuchKey, obj),
+                    HTTP_NOT_FOUND: not_found_handler,
                     HTTP_PRECONDITION_FAILED: PreconditionFailed,
                     HTTP_REQUESTED_RANGE_NOT_SATISFIABLE: InvalidRange,
                 },
@@ -986,7 +1123,7 @@ class Request(swob.Request):
                     HTTP_REQUEST_TIMEOUT: RequestTimeout,
                 },
                 'POST': {
-                    HTTP_NOT_FOUND: (NoSuchKey, obj),
+                    HTTP_NOT_FOUND: not_found_handler,
                     HTTP_PRECONDITION_FAILED: PreconditionFailed,
                 },
                 'DELETE': {
@@ -1035,7 +1172,8 @@ class Request(swob.Request):
                 self.user_id = self.access_key
 
         success_codes = self._swift_success_codes(method, container, obj)
-        error_codes = self._swift_error_codes(method, container, obj)
+        error_codes = self._swift_error_codes(method, container, obj,
+                                              sw_req.environ, app)
 
         if status in success_codes:
             return resp
@@ -1174,14 +1312,16 @@ class S3AclRequest(Request):
             self.user_id = "%s:%s" % (sw_resp.environ['HTTP_X_TENANT_NAME'],
                                       sw_resp.environ['HTTP_X_USER_NAME'])
             self.user_id = utf8encode(self.user_id)
-            self.token = sw_resp.environ['HTTP_X_AUTH_TOKEN']
-            # Need to skip S3 authorization since authtoken middleware
-            # overwrites account in PATH_INFO
-            del self.headers['Authorization']
+            self.token = sw_resp.environ.get('HTTP_X_AUTH_TOKEN')
         else:
             # tempauth
             self.user_id = self.access_key
 
+        # Need to skip S3 authorization on subsequent requests to prevent
+        # overwriting the account in PATH_INFO
+        del self.headers['Authorization']
+        del self.environ['swift3.auth_details']
+
     def to_swift_req(self, method, container, obj, query=None,
                      body=None, headers=None):
         sw_req = super(S3AclRequest, self).to_swift_req(
