@@ -100,7 +100,10 @@ class Response(ResponseBase, swob.Response):
             _key = key.lower()
 
             if _key.startswith('x-object-meta-'):
-                headers['x-amz-meta-' + _key[14:]] = val
+                # Note that AWS ignores user-defined headers with '=' in the
+                # header name. We translated underscores to '=5F' on the way
+                # in, though.
+                headers['x-amz-meta-' + _key[14:].replace('=5f', '_')] = val
             elif _key in ('content-length', 'content-type',
                           'content-range', 'content-encoding',
                           'content-disposition', 'content-language',
@@ -260,7 +263,7 @@ class BucketAlreadyOwnedByYou(ErrorResponse):
 
 class BucketNotEmpty(ErrorResponse):
     _status = '409 Conflict'
-    _msg = 'The bucket you tried to delete is not empty.'
+    _msg = 'The bucket you tried to delete is not empty'
 
 
 class CredentialsNotSupported(ErrorResponse):
@@ -604,7 +607,7 @@ class RequestTimeout(ErrorResponse):
 
 class RequestTimeTooSkewed(ErrorResponse):
     _status = '403 Forbidden'
-    _msg = 'The difference between the request time and the server\'s time ' \
+    _msg = 'The difference between the request time and the current time ' \
            'is too large.'
 
 
