@@ -40,7 +40,26 @@ class TestSwift3Bucket(Swift3FunctionalTestCase):
         self.assertEqual(status, 200)
 
         self.assertCommonResponseHeaders(headers)
-        self.assertEqual(headers['location'], '/' + bucket)
+        self.assertIn(headers['location'], (
+            '/' + bucket,  # swob won't touch it...
+            # but webob (which we get because of auth_token) *does*
+            'http://%s%s/%s' % (
+                self.conn.host,
+                '' if self.conn.port == 80 else ':%d' % self.conn.port,
+                bucket),
+            # This is all based on the Host header the client provided,
+            # and boto will double-up ports for sig v4. See
+            #   - https://github.com/boto/boto/issues/2623
+            #   - https://github.com/boto/boto/issues/3716
+            # with proposed fixes at
+            #   - https://github.com/boto/boto/pull/3513
+            #   - https://github.com/boto/boto/pull/3676
+            'http://%s%s:%d/%s' % (
+                self.conn.host,
+                '' if self.conn.port == 80 else ':%d' % self.conn.port,
+                self.conn.port,
+                bucket),
+        ))
         self.assertEqual(headers['content-length'], '0')
 
         # GET Bucket(Without Object)
@@ -55,8 +74,8 @@ class TestSwift3Bucket(Swift3FunctionalTestCase):
 
         elem = fromstring(body, 'ListBucketResult')
         self.assertEqual(elem.find('Name').text, bucket)
-        self.assertEqual(elem.find('Prefix').text, None)
-        self.assertEqual(elem.find('Marker').text, None)
+        self.assertIsNone(elem.find('Prefix').text)
+        self.assertIsNone(elem.find('Marker').text)
         self.assertEqual(elem.find('MaxKeys').text,
                          str(CONF.max_bucket_listing))
         self.assertEqual(elem.find('IsTruncated').text, 'false')
@@ -72,8 +91,8 @@ class TestSwift3Bucket(Swift3FunctionalTestCase):
 
         elem = fromstring(body, 'ListBucketResult')
         self.assertEqual(elem.find('Name').text, bucket)
-        self.assertEqual(elem.find('Prefix').text, None)
-        self.assertEqual(elem.find('Marker').text, None)
+        self.assertIsNone(elem.find('Prefix').text)
+        self.assertIsNone(elem.find('Marker').text)
         self.assertEqual(elem.find('MaxKeys').text,
                          str(CONF.max_bucket_listing))
         self.assertEqual(elem.find('IsTruncated').text, 'false')
@@ -283,6 +302,121 @@ class TestSwift3Bucket(Swift3FunctionalTestCase):
             self.assertTrue(o.find('Owner/DisplayName').text,
                             self.conn.user_id)
 
+    def test_get_bucket_v2_with_start_after(self):
+        bucket = 'bucket'
+        put_objects = ('object', 'object2', 'subdir/object', 'subdir2/object',
+                       'dir/subdir/object')
+        self._prepare_test_get_bucket(bucket, put_objects)
+
+        marker = 'object'
+        query = 'list-type=2&start-after=%s' % marker
+        expect_objects = ('object2', 'subdir/object', 'subdir2/object')
+        status, headers, body = \
+            self.conn.make_request('GET', bucket, query=query)
+        self.assertEqual(status, 200)
+        elem = fromstring(body, 'ListBucketResult')
+        self.assertEqual(elem.find('StartAfter').text, marker)
+        resp_objects = elem.findall('./Contents')
+        self.assertEqual(len(list(resp_objects)), len(expect_objects))
+        for i, o in enumerate(resp_objects):
+            self.assertEqual(o.find('Key').text, expect_objects[i])
+            self.assertTrue(o.find('LastModified').text is not None)
+            self.assertRegexpMatches(
+                o.find('LastModified').text,
+                r'^\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}\.\d{3}Z$')
+            self.assertTrue(o.find('ETag').text is not None)
+            self.assertTrue(o.find('Size').text is not None)
+            self.assertEqual(o.find('StorageClass').text, 'STANDARD')
+            self.assertIsNone(o.find('Owner/ID'))
+            self.assertIsNone(o.find('Owner/DisplayName'))
+
+    def test_get_bucket_v2_with_fetch_owner(self):
+        bucket = 'bucket'
+        put_objects = ('object', 'object2', 'subdir/object', 'subdir2/object',
+                       'dir/subdir/object')
+        self._prepare_test_get_bucket(bucket, put_objects)
+
+        query = 'list-type=2&fetch-owner=true'
+        expect_objects = ('dir/subdir/object', 'object', 'object2',
+                          'subdir/object', 'subdir2/object')
+        status, headers, body = \
+            self.conn.make_request('GET', bucket, query=query)
+        self.assertEqual(status, 200)
+        elem = fromstring(body, 'ListBucketResult')
+        self.assertEqual(elem.find('KeyCount').text, '5')
+        resp_objects = elem.findall('./Contents')
+        self.assertEqual(len(list(resp_objects)), len(expect_objects))
+        for i, o in enumerate(resp_objects):
+            self.assertEqual(o.find('Key').text, expect_objects[i])
+            self.assertTrue(o.find('LastModified').text is not None)
+            self.assertRegexpMatches(
+                o.find('LastModified').text,
+                r'^\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}\.\d{3}Z$')
+            self.assertTrue(o.find('ETag').text is not None)
+            self.assertTrue(o.find('Size').text is not None)
+            self.assertEqual(o.find('StorageClass').text, 'STANDARD')
+            self.assertTrue(o.find('Owner/ID').text, self.conn.user_id)
+            self.assertTrue(o.find('Owner/DisplayName').text,
+                            self.conn.user_id)
+
+    def test_get_bucket_v2_with_continuation_token(self):
+        bucket = 'bucket'
+        put_objects = ('object', 'object2', 'subdir/object', 'subdir2/object',
+                       'dir/subdir/object')
+        self._prepare_test_get_bucket(bucket, put_objects)
+
+        query = 'list-type=2&max-keys=3'
+        expect_objects = ('dir/subdir/object', 'object', 'object2')
+        status, headers, body = \
+            self.conn.make_request('GET', bucket, query=query)
+        self.assertEqual(status, 200)
+        elem = fromstring(body, 'ListBucketResult')
+        self.assertEqual(elem.find('MaxKeys').text, '3')
+        self.assertEqual(elem.find('KeyCount').text, '3')
+        self.assertEqual(elem.find('IsTruncated').text, 'true')
+        next_cont_token_elem = elem.find('NextContinuationToken')
+        self.assertIsNotNone(next_cont_token_elem)
+        resp_objects = elem.findall('./Contents')
+        self.assertEqual(len(list(resp_objects)), len(expect_objects))
+        for i, o in enumerate(resp_objects):
+            self.assertEqual(o.find('Key').text, expect_objects[i])
+            self.assertTrue(o.find('LastModified').text is not None)
+            self.assertRegexpMatches(
+                o.find('LastModified').text,
+                r'^\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}\.\d{3}Z$')
+            self.assertTrue(o.find('ETag').text is not None)
+            self.assertTrue(o.find('Size').text is not None)
+            self.assertEqual(o.find('StorageClass').text, 'STANDARD')
+            self.assertIsNone(o.find('Owner/ID'))
+            self.assertIsNone(o.find('Owner/DisplayName'))
+
+        query = 'list-type=2&max-keys=3&continuation-token=%s' % \
+            next_cont_token_elem.text
+        expect_objects = ('subdir/object', 'subdir2/object')
+        status, headers, body = \
+            self.conn.make_request('GET', bucket, query=query)
+        self.assertEqual(status, 200)
+        elem = fromstring(body, 'ListBucketResult')
+        self.assertEqual(elem.find('MaxKeys').text, '3')
+        self.assertEqual(elem.find('KeyCount').text, '2')
+        self.assertEqual(elem.find('IsTruncated').text, 'false')
+        self.assertIsNone(elem.find('NextContinuationToken'))
+        cont_token_elem = elem.find('ContinuationToken')
+        self.assertEqual(cont_token_elem.text, next_cont_token_elem.text)
+        resp_objects = elem.findall('./Contents')
+        self.assertEqual(len(list(resp_objects)), len(expect_objects))
+        for i, o in enumerate(resp_objects):
+            self.assertEqual(o.find('Key').text, expect_objects[i])
+            self.assertTrue(o.find('LastModified').text is not None)
+            self.assertRegexpMatches(
+                o.find('LastModified').text,
+                r'^\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}\.\d{3}Z$')
+            self.assertTrue(o.find('ETag').text is not None)
+            self.assertTrue(o.find('Size').text is not None)
+            self.assertEqual(o.find('StorageClass').text, 'STANDARD')
+            self.assertIsNone(o.find('Owner/ID'))
+            self.assertIsNone(o.find('Owner/DisplayName'))
+
     def test_head_bucket_error(self):
         self.conn.make_request('PUT', 'bucket')
 
