@@ -13,6 +13,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import base64
 import unittest
 import os
 import boto
@@ -22,15 +23,16 @@ import boto
 from distutils.version import StrictVersion
 
 from hashlib import md5
-from itertools import izip
+from itertools import izip, izip_longest
 
+from swift3.cfg import CONF
 from swift3.test.functional.utils import get_error_code, get_error_msg
 from swift3.etree import fromstring, tostring, Element, SubElement
 from swift3.test.functional import Swift3FunctionalTestCase
 from swift3.utils import mktime
 from swift3.test.functional.s3_test_client import Connection
 
-MIN_SEGMENT_SIZE = 5242880
+MIN_SEGMENT_SIZE = CONF.min_segment_size
 
 
 class TestSwift3MultiUpload(Swift3FunctionalTestCase):
@@ -46,13 +48,16 @@ class TestSwift3MultiUpload(Swift3FunctionalTestCase):
         return tostring(elem)
 
     def _initiate_multi_uploads_result_generator(self, bucket, keys,
-                                                 trials=1):
+                                                 headers=None, trials=1):
+        if headers is None:
+            headers = [None] * len(keys)
         self.conn.make_request('PUT', bucket)
         query = 'uploads'
-        for key in keys:
+        for key, key_headers in izip_longest(keys, headers):
             for i in xrange(trials):
                 status, resp_headers, body = \
-                    self.conn.make_request('POST', bucket, key, query=query)
+                    self.conn.make_request('POST', bucket, key,
+                                           headers=key_headers, query=query)
                 yield status, resp_headers, body
 
     def _upload_part(self, bucket, key, upload_id, content=None, part_num=1):
@@ -88,11 +93,14 @@ class TestSwift3MultiUpload(Swift3FunctionalTestCase):
 
     def test_object_multi_upload(self):
         bucket = 'bucket'
-        keys = ['obj1', 'obj2']
+        keys = ['obj1', 'obj2', 'obj3']
+        headers = [None,
+                   {'Content-MD5': base64.b64encode('a' * 16).strip()},
+                   {'Etag': 'nonsense'}]
         uploads = []
 
         results_generator = self._initiate_multi_uploads_result_generator(
-            bucket, keys)
+            bucket, keys, headers=headers)
 
         # Initiate Multipart Upload
         for expected_key, (status, headers, body) in \
@@ -126,14 +134,14 @@ class TestSwift3MultiUpload(Swift3FunctionalTestCase):
         self.assertEqual(headers['content-length'], str(len(body)))
         elem = fromstring(body, 'ListMultipartUploadsResult')
         self.assertEqual(elem.find('Bucket').text, bucket)
-        self.assertEqual(elem.find('KeyMarker').text, None)
+        self.assertIsNone(elem.find('KeyMarker').text)
         self.assertEqual(elem.find('NextKeyMarker').text, uploads[-1][0])
-        self.assertEqual(elem.find('UploadIdMarker').text, None)
+        self.assertIsNone(elem.find('UploadIdMarker').text)
         self.assertEqual(elem.find('NextUploadIdMarker').text, uploads[-1][1])
         self.assertEqual(elem.find('MaxUploads').text, '1000')
         self.assertTrue(elem.find('EncodingType') is None)
         self.assertEqual(elem.find('IsTruncated').text, 'false')
-        self.assertEqual(len(elem.findall('Upload')), 2)
+        self.assertEqual(len(elem.findall('Upload')), 3)
         for (expected_key, expected_upload_id), u in \
                 izip(uploads, elem.findall('Upload')):
             key = u.find('Key').text
@@ -258,17 +266,19 @@ class TestSwift3MultiUpload(Swift3FunctionalTestCase):
             self.assertEqual(MIN_SEGMENT_SIZE, int(p.find('Size').text))
             etags.append(p.find('ETag').text)
 
-        # Abort Multipart Upload
-        key, upload_id = uploads[1]
-        query = 'uploadId=%s' % upload_id
-        status, headers, body = \
-            self.conn.make_request('DELETE', bucket, key, query=query)
-        self.assertEqual(status, 204)
-        self.assertCommonResponseHeaders(headers)
-        self.assertTrue('content-type' in headers)
-        self.assertEqual(headers['content-type'], 'text/html; charset=UTF-8')
-        self.assertTrue('content-length' in headers)
-        self.assertEqual(headers['content-length'], '0')
+        # Abort Multipart Uploads
+        # note that uploads[1] has part data while uploads[2] does not
+        for key, upload_id in uploads[1:]:
+            query = 'uploadId=%s' % upload_id
+            status, headers, body = \
+                self.conn.make_request('DELETE', bucket, key, query=query)
+            self.assertEqual(status, 204)
+            self.assertCommonResponseHeaders(headers)
+            self.assertTrue('content-type' in headers)
+            self.assertEqual(headers['content-type'],
+                             'text/html; charset=UTF-8')
+            self.assertTrue('content-length' in headers)
+            self.assertEqual(headers['content-length'], '0')
 
         # Complete Multipart Upload
         key, upload_id = uploads[0]
@@ -444,6 +454,10 @@ class TestSwift3MultiUpload(Swift3FunctionalTestCase):
             self.conn.make_request('DELETE', 'nothing', key, query=query)
         self.assertEqual(get_error_code(body), 'NoSuchBucket')
 
+        status, headers, body = \
+            self.conn.make_request('DELETE', bucket, 'nothing', query=query)
+        self.assertEqual(get_error_code(body), 'NoSuchUpload')
+
         query = 'uploadId=%s' % 'nothing'
         status, headers, body = \
             self.conn.make_request('DELETE', bucket, key, query=query)
@@ -522,6 +536,110 @@ class TestSwift3MultiUpload(Swift3FunctionalTestCase):
                                    query=query)
         self.assertEqual(get_error_code(body), 'InvalidPart')
 
+    def test_complete_upload_min_segment_size(self):
+        bucket = 'bucket'
+        key = 'obj'
+        self.conn.make_request('PUT', bucket)
+        query = 'uploads'
+        status, headers, body = \
+            self.conn.make_request('POST', bucket, key, query=query)
+        elem = fromstring(body, 'InitiateMultipartUploadResult')
+        upload_id = elem.find('UploadId').text
+
+        # multi parts with no body
+        etags = []
+        for i in xrange(1, 3):
+            query = 'partNumber=%s&uploadId=%s' % (i, upload_id)
+            status, headers, body = \
+                self.conn.make_request('PUT', bucket, key, query=query)
+            etags.append(headers['etag'])
+            xml = self._gen_comp_xml(etags)
+
+        query = 'uploadId=%s' % upload_id
+        status, headers, body = \
+            self.conn.make_request('POST', bucket, key, body=xml,
+                                   query=query)
+        self.assertEqual(get_error_code(body), 'EntityTooSmall')
+
+        # multi parts with all parts less than min segment size
+        etags = []
+        for i in xrange(1, 3):
+            query = 'partNumber=%s&uploadId=%s' % (i, upload_id)
+            status, headers, body = \
+                self.conn.make_request('PUT', bucket, key, query=query,
+                                       body='AA')
+            etags.append(headers['etag'])
+            xml = self._gen_comp_xml(etags)
+
+        query = 'uploadId=%s' % upload_id
+        status, headers, body = \
+            self.conn.make_request('POST', bucket, key, body=xml,
+                                   query=query)
+        self.assertEqual(get_error_code(body), 'EntityTooSmall')
+
+        # one part and less than min segment size
+        etags = []
+        query = 'partNumber=1&uploadId=%s' % upload_id
+        status, headers, body = \
+            self.conn.make_request('PUT', bucket, key, query=query,
+                                   body='AA')
+        etags.append(headers['etag'])
+        xml = self._gen_comp_xml(etags)
+
+        query = 'uploadId=%s' % upload_id
+        status, headers, body = \
+            self.conn.make_request('POST', bucket, key, body=xml,
+                                   query=query)
+        self.assertEqual(status, 200)
+
+        # multi parts with all parts except the first part less than min
+        # segment size
+        query = 'uploads'
+        status, headers, body = \
+            self.conn.make_request('POST', bucket, key, query=query)
+        elem = fromstring(body, 'InitiateMultipartUploadResult')
+        upload_id = elem.find('UploadId').text
+
+        etags = []
+        body_size = [MIN_SEGMENT_SIZE, MIN_SEGMENT_SIZE - 1, 2]
+        for i in xrange(1, 3):
+            query = 'partNumber=%s&uploadId=%s' % (i, upload_id)
+            status, headers, body = \
+                self.conn.make_request('PUT', bucket, key, query=query,
+                                       body='A' * body_size[i])
+            etags.append(headers['etag'])
+            xml = self._gen_comp_xml(etags)
+
+        query = 'uploadId=%s' % upload_id
+        status, headers, body = \
+            self.conn.make_request('POST', bucket, key, body=xml,
+                                   query=query)
+        self.assertEqual(get_error_code(body), 'EntityTooSmall')
+
+        # multi parts with all parts except last part more than min segment
+        # size
+        query = 'uploads'
+        status, headers, body = \
+            self.conn.make_request('POST', bucket, key, query=query)
+        elem = fromstring(body, 'InitiateMultipartUploadResult')
+        upload_id = elem.find('UploadId').text
+
+        etags = []
+        body_size = [MIN_SEGMENT_SIZE, MIN_SEGMENT_SIZE, 2]
+        for i in xrange(1, 3):
+            query = 'partNumber=%s&uploadId=%s' % (i, upload_id)
+            status, headers, body = \
+                self.conn.make_request('PUT', bucket, key, query=query,
+                                       body='A' * body_size[i])
+            etags.append(headers['etag'])
+            xml = self._gen_comp_xml(etags)
+
+        query = 'uploadId=%s' % upload_id
+        status, headers, body = \
+            self.conn.make_request('POST', bucket, key, body=xml,
+                                   query=query)
+        self.assertEqual(status, 200)
+
     def test_complete_upload_with_fewer_etags(self):
         bucket = 'bucket'
         key = 'obj'
