@@ -194,9 +194,7 @@ class TestSwift3Object(Swift3FunctionalTestCase):
         self.assertEqual(headers['content-type'], 'application/xml')
 
         status, headers, body = self.conn.make_request('GET', 'invalid', obj)
-        # TODO; requires consideration
-        # self.assertEqual(get_error_code(body), 'NoSuchBucket')
-        self.assertEqual(get_error_code(body), 'NoSuchKey')
+        self.assertEqual(get_error_code(body), 'NoSuchBucket')
         self.assertEqual(headers['content-type'], 'application/xml')
 
     def test_head_object_error(self):
@@ -320,7 +318,9 @@ class TestSwift3Object(Swift3FunctionalTestCase):
         self.assertCommonResponseHeaders(headers)
         self._assertObjectEtag(self.bucket, obj, etag)
 
-    def _test_put_object_headers(self, req_headers):
+    def _test_put_object_headers(self, req_headers, expected_headers=None):
+        if expected_headers is None:
+            expected_headers = req_headers
         obj = 'object'
         content = 'abcdefghij'
         etag = md5(content).hexdigest()
@@ -330,7 +330,7 @@ class TestSwift3Object(Swift3FunctionalTestCase):
         self.assertEqual(status, 200)
         status, headers, body = \
             self.conn.make_request('HEAD', self.bucket, obj)
-        for header, value in req_headers.items():
+        for header, value in expected_headers.items():
             self.assertIn(header.lower(), headers)
             self.assertEqual(headers[header.lower()], value)
         self.assertCommonResponseHeaders(headers)
@@ -341,6 +341,21 @@ class TestSwift3Object(Swift3FunctionalTestCase):
             'X-Amz-Meta-Bar': 'foo',
             'X-Amz-Meta-Bar2': 'foo2'})
 
+    def test_put_object_weird_metadata(self):
+        req_headers = dict(
+            ('x-amz-meta-' + c, c)
+            for c in '!"#$%&\'()*+-./<=>?@[\\]^`{|}~')
+        exp_headers = dict(
+            ('x-amz-meta-' + c, c)
+            for c in '!#$%&\'(*+-.^`|~')
+        self._test_put_object_headers(req_headers, exp_headers)
+
+    def test_put_object_underscore_in_metadata(self):
+        # Break this out separately for ease of testing pre-0.19.0 eventlet
+        self._test_put_object_headers({
+            'X-Amz-Meta-Foo-Bar': 'baz',
+            'X-Amz-Meta-Foo_Bar': 'also baz'})
+
     def test_put_object_content_headers(self):
         self._test_put_object_headers({
             'Content-Type': 'foo/bar',
@@ -372,6 +387,41 @@ class TestSwift3Object(Swift3FunctionalTestCase):
         self.assertCommonResponseHeaders(headers)
         self._assertObjectEtag(self.bucket, obj, etag)
 
+    def test_put_object_copy_source_params(self):
+        obj = 'object'
+        src_headers = {'X-Amz-Meta-Test': 'src'}
+        src_body = 'some content'
+        dst_bucket = 'dst-bucket'
+        dst_obj = 'dst_object'
+        self.conn.make_request('PUT', self.bucket, obj, src_headers, src_body)
+        self.conn.make_request('PUT', dst_bucket)
+
+        headers = {'X-Amz-Copy-Source': '/%s/%s?nonsense' % (
+            self.bucket, obj)}
+        status, headers, body = \
+            self.conn.make_request('PUT', dst_bucket, dst_obj, headers)
+        self.assertEqual(status, 400)
+        self.assertEqual(get_error_code(body), 'InvalidArgument')
+
+        headers = {'X-Amz-Copy-Source': '/%s/%s?versionId=null&nonsense' % (
+            self.bucket, obj)}
+        status, headers, body = \
+            self.conn.make_request('PUT', dst_bucket, dst_obj, headers)
+        self.assertEqual(status, 400)
+        self.assertEqual(get_error_code(body), 'InvalidArgument')
+
+        headers = {'X-Amz-Copy-Source': '/%s/%s?versionId=null' % (
+            self.bucket, obj)}
+        status, headers, body = \
+            self.conn.make_request('PUT', dst_bucket, dst_obj, headers)
+        self.assertEqual(status, 200)
+        self.assertCommonResponseHeaders(headers)
+        status, headers, body = \
+            self.conn.make_request('GET', dst_bucket, dst_obj)
+        self.assertEqual(status, 200)
+        self.assertEqual(headers['x-amz-meta-test'], 'src')
+        self.assertEqual(body, src_body)
+
     def test_put_object_copy_source(self):
         obj = 'object'
         content = 'abcdefghij'
