@@ -14,21 +14,32 @@
 # limitations under the License.
 
 import unittest
-from mock import patch
+from mock import patch, MagicMock
 from contextlib import nested
 from datetime import datetime
 import hashlib
-import base64
+import mock
+import requests
+import json
+import copy
 from urllib import unquote, quote
 
+from swift.common.middleware.keystoneauth import KeystoneAuth
 from swift.common import swob, utils
 from swift.common.swob import Request
 
+from keystonemiddleware.auth_token import AuthProtocol
+from keystoneauth1.access import AccessInfoV2
+
 import swift3
 from swift3.test.unit import Swift3TestCase
+from swift3.test.unit.helpers import FakeSwift
+from swift3.test.unit.test_s3_token_middleware import \
+    GOOD_RESPONSE_V2, GOOD_RESPONSE_V3
 from swift3.request import SigV4Request, Request as S3Request
 from swift3.etree import fromstring
-from swift3.middleware import filter_factory
+from swift3.middleware import filter_factory, Swift3Middleware
+from swift3.s3_token_middleware import S3Token
 from swift3.cfg import CONF
 
 
@@ -92,16 +103,21 @@ class TestSwift3Middleware(Swift3TestCase):
                 path, query_string = path.split('?', 1)
             else:
                 query_string = ''
+            env = {
+                'REQUEST_METHOD': 'GET',
+                'PATH_INFO': path,
+                'QUERY_STRING': query_string,
+                'HTTP_AUTHORIZATION': 'AWS X:Y:Z',
+            }
+            for header, value in headers.items():
+                header = 'HTTP_' + header.replace('-', '_').upper()
+                if header in ('HTTP_CONTENT_TYPE', 'HTTP_CONTENT_LENGTH'):
+                    header = header[5:]
+                env[header] = value
 
             with patch('swift3.request.Request._validate_headers'):
-                req = S3Request({
-                    'REQUEST_METHOD': 'GET',
-                    'PATH_INFO': path,
-                    'QUERY_STRING': query_string,
-                    'HTTP_AUTHORIZATION': 'AWS X:Y:Z',
-                })
-            req.headers.update(headers)
-            return req._string_to_sign()
+                req = S3Request(env)
+            return req.environ['swift3.auth_details']['string_to_sign']
 
         def verify(hash, path, headers):
             s = canonical_string(path, headers)
@@ -163,6 +179,19 @@ class TestSwift3Middleware(Swift3TestCase):
         self.assertEqual(str1, str2)
         self.assertEqual(str2, str3)
 
+        # Note that boto does not do proper stripping (as of 2.42.0).
+        # These were determined by examining the StringToSignBytes element of
+        # resulting SignatureDoesNotMatch errors from AWS.
+        str1 = canonical_string('/', {'Content-Type': 'text/plain',
+                                      'Content-MD5': '##'})
+        str2 = canonical_string('/', {'Content-Type': '\x01\x02text/plain',
+                                      'Content-MD5': '\x1f ##'})
+        str3 = canonical_string('/', {'Content-Type': 'text/plain \x10',
+                                      'Content-MD5': '##\x18'})
+
+        self.assertEqual(str1, str2)
+        self.assertEqual(str2, str3)
+
     def test_signed_urls_expired(self):
         expire = '1000000000'
         req = Request.blank('/bucket/object?Signature=X&Expires=%s&'
@@ -354,12 +383,16 @@ class TestSwift3Middleware(Swift3TestCase):
         req.headers['Authorization'] = 'AWS test:tester:hmac'
         date_header = self.get_date_header()
         req.headers['Date'] = date_header
-        status, headers, body = self.call_swift3(req)
+        with mock.patch('swift3.request.Request.check_signature') as mock_cs:
+            status, headers, body = self.call_swift3(req)
         _, _, headers = self.swift.calls_with_headers[-1]
-        self.assertEqual(base64.urlsafe_b64decode(
-            headers['X-Auth-Token']),
-            'PUT\n\n\n%s\n/bucket/object?partNumber=1&uploadId=123456789abcdef'
-            % date_header)
+        self.assertEqual(req.environ['swift3.auth_details'], {
+            'access_key': 'test:tester',
+            'signature': 'hmac',
+            'string_to_sign': '\n'.join([
+                'PUT', '', '', date_header,
+                '/bucket/object?partNumber=1&uploadId=123456789abcdef']),
+            'check_signature': mock_cs})
 
     def test_invalid_uri(self):
         req = Request.blank('/bucket/invalid\xffname',
@@ -508,19 +541,24 @@ class TestSwift3Middleware(Swift3TestCase):
             pipeline.return_value = 'swift3 tempauth proxy-server'
             self.swift3.check_pipeline(conf)
 
+            # This *should* still work; authtoken will remove our auth details,
+            # but the X-Auth-Token we drop in will remain
+            # if we found one in the response
             pipeline.return_value = 'swift3 s3token authtoken keystoneauth ' \
                 'proxy-server'
             self.swift3.check_pipeline(conf)
 
+            # This should work now; no more doubled-up requests to keystone!
+            pipeline.return_value = 'swift3 s3token keystoneauth proxy-server'
+            self.swift3.check_pipeline(conf)
+
             pipeline.return_value = 'swift3 swauth proxy-server'
             self.swift3.check_pipeline(conf)
 
+            # Note that authtoken would need to have delay_auth_decision=True
             pipeline.return_value = 'swift3 authtoken s3token keystoneauth ' \
                 'proxy-server'
-            with self.assertRaises(ValueError) as cm:
-                self.swift3.check_pipeline(conf)
-            self.assertIn('expected filter s3token before authtoken before '
-                          'keystoneauth', cm.exception.message)
+            self.swift3.check_pipeline(conf)
 
             pipeline.return_value = 'swift3 proxy-server'
             with self.assertRaises(ValueError) as cm:
@@ -652,7 +690,7 @@ class TestSwift3Middleware(Swift3TestCase):
         test(auth_str, 'AccessDenied', 'Access Denied.')
 
     def test_canonical_string_v4(self):
-        def canonical_string(path, environ):
+        def _get_req(path, environ):
             if '?' in path:
                 path, query_string = path.split('?', 1)
             else:
@@ -663,19 +701,34 @@ class TestSwift3Middleware(Swift3TestCase):
                 'PATH_INFO': path,
                 'QUERY_STRING': query_string,
                 'HTTP_DATE': 'Mon, 09 Sep 2011 23:36:00 GMT',
-                'HTTP_X_AMZ_CONTENT_SHA256': (
+                'HTTP_X_AMZ_CONTENT_SHA256':
                     'e3b0c44298fc1c149afbf4c8996fb924'
-                    '27ae41e4649b934ca495991b7852b855')
+                    '27ae41e4649b934ca495991b7852b855',
+                'HTTP_AUTHORIZATION':
+                    'AWS4-HMAC-SHA256 '
+                    'Credential=X:Y/dt/reg/host/blah, '
+                    'SignedHeaders=content-md5;content-type;date, '
+                    'Signature=x',
             }
             env.update(environ)
             with patch('swift3.request.Request._validate_headers'):
                 req = SigV4Request(env)
-            return req._string_to_sign()
+            return req
+
+        def canonical_string(path, environ):
+            return _get_req(path, environ)._canonical_request()
 
         def verify(hash_val, path, environ):
-            s = canonical_string(path, environ)
-            s = s.split('\n')[3]
-            self.assertEqual(hash_val, s)
+            # See http://docs.aws.amazon.com/general/latest/gr
+            # /signature-v4-test-suite.html for where location, service, and
+            # signing key came from
+            with patch.object(CONF, 'location', 'us-east-1'), \
+                    patch.object(swift3.request, 'SERVICE', 'host'):
+                req = _get_req(path, environ)
+                hash_in_sts = req._string_to_sign().split('\n')[3]
+                self.assertEqual(hash_val, hash_in_sts)
+                self.assertTrue(req.check_signature(
+                    'wJalrXUtnFEMI/K7MDENG+bPxRfiCYEXAMPLEKEY'))
 
         # all next data got from aws4_testsuite from Amazon
         # http://docs.aws.amazon.com/general/latest/gr/samples
@@ -688,7 +741,9 @@ class TestSwift3Middleware(Swift3TestCase):
             'HTTP_AUTHORIZATION': (
                 'AWS4-HMAC-SHA256 '
                 'Credential=AKIDEXAMPLE/20110909/us-east-1/host/aws4_request, '
-                'SignedHeaders=date;host, Signature=X'),
+                'SignedHeaders=date;host, '
+                'Signature=b27ccfbfa7df52a200ff74193ca6e32d'
+                '4b48b8856fab7ebf1c595d0670a7e470'),
             'HTTP_HOST': 'host.foo.com'}
         verify('366b91fb121d72a00f46bbe8d395f53a'
                '102b06dfb7e79636515208ed3fa606b1',
@@ -700,7 +755,9 @@ class TestSwift3Middleware(Swift3TestCase):
             'HTTP_AUTHORIZATION': (
                 'AWS4-HMAC-SHA256 '
                 'Credential=AKIDEXAMPLE/20110909/us-east-1/host/aws4_request, '
-                'SignedHeaders=date;host;p, Signature=X'),
+                'SignedHeaders=date;host;p, '
+                'Signature=debf546796015d6f6ded8626f5ce9859'
+                '7c33b47b9164cf6b17b4642036fcb592'),
             'HTTP_HOST': 'host.foo.com',
             'HTTP_P': 'phfft'}
         verify('dddd1902add08da1ac94782b05f9278c'
@@ -712,7 +769,9 @@ class TestSwift3Middleware(Swift3TestCase):
             'HTTP_AUTHORIZATION': (
                 'AWS4-HMAC-SHA256 '
                 'Credential=AKIDEXAMPLE/20110909/us-east-1/host/aws4_request, '
-                'SignedHeaders=date;host, Signature=X'),
+                'SignedHeaders=date;host, '
+                'Signature=8d6634c189aa8c75c2e51e106b6b5121'
+                'bed103fdb351f7d7d4381c738823af74'),
             'HTTP_HOST': 'host.foo.com',
             'RAW_PATH_INFO': '/%E1%88%B4'}
 
@@ -734,7 +793,9 @@ class TestSwift3Middleware(Swift3TestCase):
             'HTTP_AUTHORIZATION': (
                 'AWS4-HMAC-SHA256 '
                 'Credential=AKIDEXAMPLE/20110909/us-east-1/host/aws4_request, '
-                'SignedHeaders=date;host, Signature=X'),
+                'SignedHeaders=date;host, '
+                'Signature=0dc122f3b28b831ab48ba65cb47300de'
+                '53fbe91b577fe113edac383730254a3b'),
             'HTTP_HOST': 'host.foo.com'}
         verify('2f23d14fe13caebf6dfda346285c6d9c'
                '14f49eaca8f5ec55c627dd7404f7a727',
@@ -746,7 +807,9 @@ class TestSwift3Middleware(Swift3TestCase):
             'HTTP_AUTHORIZATION': (
                 'AWS4-HMAC-SHA256 '
                 'Credential=AKIDEXAMPLE/20110909/us-east-1/host/aws4_request, '
-                'SignedHeaders=date;host;zoo, Signature=X'),
+                'SignedHeaders=date;host;zoo, '
+                'Signature=273313af9d0c265c531e11db70bbd653'
+                'f3ba074c1009239e8559d3987039cad7'),
             'HTTP_HOST': 'host.foo.com',
             'HTTP_ZOO': 'ZOOBAR'}
         verify('3aae6d8274b8c03e2cc96fc7d6bda4b9'
@@ -759,7 +822,9 @@ class TestSwift3Middleware(Swift3TestCase):
             'HTTP_AUTHORIZATION': (
                 'AWS4-HMAC-SHA256 '
                 'Credential=AKIDEXAMPLE/20110909/us-east-1/host/aws4_request, '
-                'SignedHeaders=date;host;content-type, Signature=X'),
+                'SignedHeaders=date;host;content-type, '
+                'Signature=b105eb10c6d318d2294de9d49dd8b031'
+                'b55e3c3fe139f2e637da70511e9e7b71'),
             'HTTP_HOST': 'host.foo.com',
             'HTTP_X_AMZ_CONTENT_SHA256':
                 '3ba8907e7a252327488df390ed517c45'
@@ -776,7 +841,9 @@ class TestSwift3Middleware(Swift3TestCase):
             'HTTP_AUTHORIZATION': (
                 'AWS4-HMAC-SHA256 '
                 'Credential=AKIDEXAMPLE/20110909/us-east-1/host/aws4_request, '
-                'SignedHeaders=date;host;content-type, Signature=X'),
+                'SignedHeaders=date;host;content-type, '
+                'Signature=5a15b22cf462f047318703b92e6f4f38'
+                '884e4a7ab7b1d6426ca46a8bd1c26cbc'),
             'HTTP_HOST': 'host.foo.com',
             'HTTP_X_AMZ_CONTENT_SHA256':
                 '3ba8907e7a252327488df390ed517c45'
@@ -787,6 +854,19 @@ class TestSwift3Middleware(Swift3TestCase):
                '1b14f04345cbfe6e739236c76dd48f74',
                '/', env)
 
+        # Note that boto does not do proper stripping (as of 2.42.0).
+        # These were determined by examining the StringToSignBytes element of
+        # resulting SignatureDoesNotMatch errors from AWS.
+        str1 = canonical_string('/', {'CONTENT_TYPE': 'text/plain',
+                                      'HTTP_CONTENT_MD5': '##'})
+        str2 = canonical_string('/', {'CONTENT_TYPE': '\x01\x02text/plain',
+                                      'HTTP_CONTENT_MD5': '\x1f ##'})
+        str3 = canonical_string('/', {'CONTENT_TYPE': 'text/plain \x10',
+                                      'HTTP_CONTENT_MD5': '##\x18'})
+
+        self.assertEqual(str1, str2)
+        self.assertEqual(str2, str3)
+
     def test_mixture_param_v4(self):
         # now we have an Authorization header
         headers = {
@@ -824,6 +904,146 @@ class TestSwift3Middleware(Swift3TestCase):
         status, headers, body = self.call_swift3(req)
         self.assertEqual(status.split()[0], '403', body)
 
+    def test_swift3_with_only_s3_token(self):
+        self.swift = FakeSwift()
+        self.keystone_auth = KeystoneAuth(
+            self.swift, {'operator_roles': 'swift-user'})
+        self.s3_token = S3Token(
+            self.keystone_auth, {'auth_uri': 'https://fakehost/identity'})
+        self.swift3 = Swift3Middleware(self.s3_token, CONF)
+        req = Request.blank(
+            '/bucket',
+            environ={'REQUEST_METHOD': 'PUT'},
+            headers={'Authorization': 'AWS access:signature',
+                     'Date': self.get_date_header()})
+        self.swift.register('PUT', '/v1/AUTH_TENANT_ID/bucket',
+                            swob.HTTPCreated, {}, None)
+        self.swift.register('HEAD', '/v1/AUTH_TENANT_ID',
+                            swob.HTTPOk, {}, None)
+        with patch.object(self.s3_token, '_json_request') as mock_req:
+            mock_resp = requests.Response()
+            mock_resp._content = json.dumps(GOOD_RESPONSE_V2)
+            mock_resp.status_code = 201
+            mock_req.return_value = mock_resp
+
+            status, headers, body = self.call_swift3(req)
+            self.assertEqual(body, '')
+            self.assertEqual(1, mock_req.call_count)
+
+    def test_swift3_with_only_s3_token_v3(self):
+        self.swift = FakeSwift()
+        self.keystone_auth = KeystoneAuth(
+            self.swift, {'operator_roles': 'swift-user'})
+        self.s3_token = S3Token(
+            self.keystone_auth, {'auth_uri': 'https://fakehost/identity'})
+        self.swift3 = Swift3Middleware(self.s3_token, CONF)
+        req = Request.blank(
+            '/bucket',
+            environ={'REQUEST_METHOD': 'PUT'},
+            headers={'Authorization': 'AWS access:signature',
+                     'Date': self.get_date_header()})
+        self.swift.register('PUT', '/v1/AUTH_PROJECT_ID/bucket',
+                            swob.HTTPCreated, {}, None)
+        self.swift.register('HEAD', '/v1/AUTH_PROJECT_ID',
+                            swob.HTTPOk, {}, None)
+        with patch.object(self.s3_token, '_json_request') as mock_req:
+            mock_resp = requests.Response()
+            mock_resp._content = json.dumps(GOOD_RESPONSE_V3)
+            mock_resp.status_code = 200
+            mock_req.return_value = mock_resp
+
+            status, headers, body = self.call_swift3(req)
+            self.assertEqual(body, '')
+            self.assertEqual(1, mock_req.call_count)
+
+    def test_swift3_with_s3_token_and_auth_token(self):
+        self.swift = FakeSwift()
+        self.keystone_auth = KeystoneAuth(
+            self.swift, {'operator_roles': 'swift-user'})
+        self.auth_token = AuthProtocol(
+            self.keystone_auth, {'delay_auth_decision': 'True'})
+        self.s3_token = S3Token(
+            self.auth_token, {'auth_uri': 'https://fakehost/identity'})
+        self.swift3 = Swift3Middleware(self.s3_token, CONF)
+        req = Request.blank(
+            '/bucket',
+            environ={'REQUEST_METHOD': 'PUT'},
+            headers={'Authorization': 'AWS access:signature',
+                     'Date': self.get_date_header()})
+        self.swift.register('PUT', '/v1/AUTH_TENANT_ID/bucket',
+                            swob.HTTPCreated, {}, None)
+        self.swift.register('HEAD', '/v1/AUTH_TENANT_ID',
+                            swob.HTTPOk, {}, None)
+        with patch.object(self.s3_token, '_json_request') as mock_req:
+            with patch.object(self.auth_token,
+                              '_do_fetch_token') as mock_fetch:
+                mock_resp = requests.Response()
+                mock_resp._content = json.dumps(GOOD_RESPONSE_V2)
+                mock_resp.status_code = 201
+                mock_req.return_value = mock_resp
+
+                mock_access_info = AccessInfoV2(GOOD_RESPONSE_V2)
+                mock_access_info.will_expire_soon = \
+                    lambda stale_duration: False
+                mock_fetch.return_value = (MagicMock(), mock_access_info)
+
+                status, headers, body = self.call_swift3(req)
+                self.assertEqual(body, '')
+                self.assertEqual(1, mock_req.call_count)
+                # With X-Auth-Token, auth_token will call _do_fetch_token to
+                # connect to keystone in auth_token, again
+                self.assertEqual(1, mock_fetch.call_count)
+
+    def test_swift3_with_s3_token_no_pass_token_to_auth_token(self):
+        self.swift = FakeSwift()
+        self.keystone_auth = KeystoneAuth(
+            self.swift, {'operator_roles': 'swift-user'})
+        self.auth_token = AuthProtocol(
+            self.keystone_auth, {'delay_auth_decision': 'True'})
+        self.s3_token = S3Token(
+            self.auth_token, {'auth_uri': 'https://fakehost/identity'})
+        self.swift3 = Swift3Middleware(self.s3_token, CONF)
+        req = Request.blank(
+            '/bucket',
+            environ={'REQUEST_METHOD': 'PUT'},
+            headers={'Authorization': 'AWS access:signature',
+                     'Date': self.get_date_header()})
+        self.swift.register('PUT', '/v1/AUTH_TENANT_ID/bucket',
+                            swob.HTTPCreated, {}, None)
+        self.swift.register('HEAD', '/v1/AUTH_TENANT_ID',
+                            swob.HTTPOk, {}, None)
+        with patch.object(self.s3_token, '_json_request') as mock_req:
+            with patch.object(self.auth_token,
+                              '_do_fetch_token') as mock_fetch:
+                mock_resp = requests.Response()
+                no_token_id_good_resp = copy.deepcopy(GOOD_RESPONSE_V2)
+                # delete token id
+                del no_token_id_good_resp['access']['token']['id']
+                mock_resp._content = json.dumps(no_token_id_good_resp)
+                mock_resp.status_code = 201
+                mock_req.return_value = mock_resp
+
+                mock_access_info = AccessInfoV2(GOOD_RESPONSE_V2)
+                mock_access_info.will_expire_soon = \
+                    lambda stale_duration: False
+                mock_fetch.return_value = (MagicMock(), mock_access_info)
+
+                status, headers, body = self.call_swift3(req)
+                # No token provided from keystone result in 401 Unauthorized
+                # at `swift.common.middleware.keystoneauth` because auth_token
+                # will remove all auth headers including 'X-Identity-Status'[1]
+                # and then, set X-Identity-Status: Invalid at [2]
+                #
+                # 1: https://github.com/openstack/keystonemiddleware/blob/
+                #    master/keystonemiddleware/auth_token/__init__.py#L620
+                # 2: https://github.com/openstack/keystonemiddleware/blob/
+                #    master/keystonemiddleware/auth_token/__init__.py#L627-L629
+
+                self.assertEqual('403 Forbidden', status)
+                self.assertEqual(1, mock_req.call_count)
+                # if no token provided from keystone, we can skip the call to
+                # fetch the token
+                self.assertEqual(0, mock_fetch.call_count)
 
 if __name__ == '__main__':
     unittest.main()
