@@ -13,10 +13,12 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import base64
+from hashlib import md5
+from mock import patch
 import os
 import time
 import unittest
-from mock import patch
 from urllib import quote
 
 from swift.common import swob
@@ -24,6 +26,7 @@ from swift.common.swob import Request
 from swift.common.utils import json
 
 from swift3.test.unit import Swift3TestCase
+from swift3.test.unit.helpers import UnreadableInput
 from swift3.etree import fromstring, tostring
 from swift3.subresource import Owner, Grant, User, ACL, encode_acl, \
     decode_acl, ACLPublicRead
@@ -72,6 +75,8 @@ class TestSwift3MultiUpload(Swift3TestCase):
         self.last_modified = 'Fri, 01 Apr 2014 12:00:00 GMT'
         put_headers = {'etag': self.etag, 'last-modified': self.last_modified}
 
+        CONF.min_segment_size = 1
+
         objects = map(lambda item: {'name': item[0], 'last_modified': item[1],
                                     'hash': item[2], 'bytes': item[3]},
                       objects_template)
@@ -82,8 +87,12 @@ class TestSwift3MultiUpload(Swift3TestCase):
         self.swift.register('GET', segment_bucket, swob.HTTPOk, {},
                             object_list)
         self.swift.register('HEAD', segment_bucket + '/object/X',
-                            swob.HTTPOk, {'x-object-meta-foo': 'bar',
-                                          'content-type': 'baz/quux'}, None)
+                            swob.HTTPOk,
+                            {'x-object-meta-foo': 'bar',
+                             'content-type': 'application/directory',
+                             'x-object-sysmeta-swift3-has-content-type': 'yes',
+                             'x-object-sysmeta-swift3-content-type':
+                             'baz/quux'}, None)
         self.swift.register('PUT', segment_bucket + '/object/X',
                             swob.HTTPCreated, {}, None)
         self.swift.register('DELETE', segment_bucket + '/object/X',
@@ -97,32 +106,6 @@ class TestSwift3MultiUpload(Swift3TestCase):
         self.swift.register('DELETE', segment_bucket + '/object/X/2',
                             swob.HTTPNoContent, {}, None)
 
-        self.swift.register('HEAD', segment_bucket + '/object/Y',
-                            swob.HTTPOk, {}, None)
-        self.swift.register('PUT', segment_bucket + '/object/Y',
-                            swob.HTTPCreated, {}, None)
-        self.swift.register('DELETE', segment_bucket + '/object/Y',
-                            swob.HTTPNoContent, {}, None)
-        self.swift.register('PUT', segment_bucket + '/object/Y/1',
-                            swob.HTTPCreated, {}, None)
-        self.swift.register('DELETE', segment_bucket + '/object/Y/1',
-                            swob.HTTPNoContent, {}, None)
-        self.swift.register('DELETE', segment_bucket + '/object/Y/2',
-                            swob.HTTPNoContent, {}, None)
-
-        self.swift.register('HEAD', segment_bucket + '/object2/Z',
-                            swob.HTTPOk, {}, None)
-        self.swift.register('PUT', segment_bucket + '/object2/Z',
-                            swob.HTTPCreated, {}, None)
-        self.swift.register('DELETE', segment_bucket + '/object2/Z',
-                            swob.HTTPNoContent, {}, None)
-        self.swift.register('PUT', segment_bucket + '/object2/Z/1',
-                            swob.HTTPCreated, {}, None)
-        self.swift.register('DELETE', segment_bucket + '/object2/Z/1',
-                            swob.HTTPNoContent, {}, None)
-        self.swift.register('DELETE', segment_bucket + '/object2/Z/2',
-                            swob.HTTPNoContent, {}, None)
-
     @s3acl
     def test_bucket_upload_part(self):
         req = Request.blank('/bucket?partNumber=1&uploadId=x',
@@ -200,8 +183,8 @@ class TestSwift3MultiUpload(Swift3TestCase):
         status, headers, body = self._test_bucket_multipart_uploads_GET()
         elem = fromstring(body, 'ListMultipartUploadsResult')
         self.assertEqual(elem.find('Bucket').text, 'bucket')
-        self.assertEqual(elem.find('KeyMarker').text, None)
-        self.assertEqual(elem.find('UploadIdMarker').text, None)
+        self.assertIsNone(elem.find('KeyMarker').text)
+        self.assertIsNone(elem.find('UploadIdMarker').text)
         self.assertEqual(elem.find('NextUploadIdMarker').text, 'Z')
         self.assertEqual(elem.find('MaxUploads').text, '1000')
         self.assertEqual(elem.find('IsTruncated').text, 'false')
@@ -234,9 +217,9 @@ class TestSwift3MultiUpload(Swift3TestCase):
         self.assertEqual(status.split()[0], '200')
         elem = fromstring(body, 'ListMultipartUploadsResult')
         self.assertEqual(elem.find('Bucket').text, 'bucket')
-        self.assertEqual(elem.find('KeyMarker').text, None)
-        self.assertEqual(elem.find('UploadIdMarker').text, None)
-        self.assertEqual(elem.find('NextUploadIdMarker').text, None)
+        self.assertIsNone(elem.find('KeyMarker').text)
+        self.assertIsNone(elem.find('UploadIdMarker').text)
+        self.assertIsNone(elem.find('NextUploadIdMarker').text)
         self.assertEqual(elem.find('MaxUploads').text, '1000')
         self.assertEqual(elem.find('IsTruncated').text, 'false')
         self.assertEqual(len(elem.findall('Upload')), 0)
@@ -545,23 +528,62 @@ class TestSwift3MultiUpload(Swift3TestCase):
         self.assertTrue(query.get('delimiter') is None)
 
     @patch('swift3.controllers.multi_upload.unique_id', lambda: 'X')
-    def test_object_multipart_upload_initiate(self):
+    def _test_object_multipart_upload_initiate(self, headers):
+        headers.update({
+            'Authorization': 'AWS test:tester:hmac',
+            'Date': self.get_date_header(),
+            'x-amz-meta-foo': 'bar',
+        })
         req = Request.blank('/bucket/object?uploads',
                             environ={'REQUEST_METHOD': 'POST'},
-                            headers={'Authorization':
-                                     'AWS test:tester:hmac',
-                                     'Date': self.get_date_header(),
-                                     'x-amz-meta-foo': 'bar'})
+                            headers=headers)
         status, headers, body = self.call_swift3(req)
         fromstring(body, 'InitiateMultipartUploadResult')
         self.assertEqual(status.split()[0], '200')
 
         _, _, req_headers = self.swift.calls_with_headers[-1]
         self.assertEqual(req_headers.get('X-Object-Meta-Foo'), 'bar')
+        self.assertNotIn('Etag', req_headers)
+        self.assertNotIn('Content-MD5', req_headers)
+
+    def test_object_multipart_upload_initiate(self):
+        self._test_object_multipart_upload_initiate({})
+        self._test_object_multipart_upload_initiate({'Etag': 'blahblahblah'})
+        self._test_object_multipart_upload_initiate({
+            'Content-MD5': base64.b64encode('blahblahblahblah').strip()})
 
     @s3acl(s3acl_only=True)
     @patch('swift3.controllers.multi_upload.unique_id', lambda: 'X')
     def test_object_multipart_upload_initiate_s3acl(self):
+        req = Request.blank('/bucket/object?uploads',
+                            environ={'REQUEST_METHOD': 'POST'},
+                            headers={'Authorization':
+                                     'AWS test:tester:hmac',
+                                     'Date': self.get_date_header(),
+                                     'x-amz-acl': 'public-read',
+                                     'x-amz-meta-foo': 'bar',
+                                     'Content-Type': 'cat/picture'})
+        status, headers, body = self.call_swift3(req)
+        fromstring(body, 'InitiateMultipartUploadResult')
+        self.assertEqual(status.split()[0], '200')
+
+        _, _, req_headers = self.swift.calls_with_headers[-1]
+        self.assertEqual(req_headers.get('X-Object-Meta-Foo'), 'bar')
+        self.assertEqual(req_headers.get(
+            'X-Object-Sysmeta-Swift3-Has-Content-Type'), 'yes')
+        self.assertEqual(req_headers.get(
+            'X-Object-Sysmeta-Swift3-Content-Type'), 'cat/picture')
+        tmpacl_header = req_headers.get(sysmeta_header('object', 'tmpacl'))
+        self.assertTrue(tmpacl_header)
+        acl_header = encode_acl('object',
+                                ACLPublicRead(Owner('test:tester',
+                                                    'test:tester')))
+        self.assertEqual(acl_header.get(sysmeta_header('object', 'acl')),
+                         tmpacl_header)
+
+    @s3acl(s3acl_only=True)
+    @patch('swift3.controllers.multi_upload.unique_id', lambda: 'X')
+    def test_object_multipart_upload_initiate_no_content_type(self):
         req = Request.blank('/bucket/object?uploads',
                             environ={'REQUEST_METHOD': 'POST'},
                             headers={'Authorization':
@@ -575,6 +597,8 @@ class TestSwift3MultiUpload(Swift3TestCase):
 
         _, _, req_headers = self.swift.calls_with_headers[-1]
         self.assertEqual(req_headers.get('X-Object-Meta-Foo'), 'bar')
+        self.assertEqual(req_headers.get(
+            'X-Object-Sysmeta-Swift3-Has-Content-Type'), 'no')
         tmpacl_header = req_headers.get(sysmeta_header('object', 'tmpacl'))
         self.assertTrue(tmpacl_header)
         acl_header = encode_acl('object',
@@ -634,6 +658,58 @@ class TestSwift3MultiUpload(Swift3TestCase):
         self.assertEqual(headers.get('X-Object-Meta-Foo'), 'bar')
         self.assertEqual(headers.get('Content-Type'), 'baz/quux')
 
+    def test_object_multipart_upload_complete_404_on_marker_delete(self):
+        segment_bucket = '/v1/AUTH_test/bucket+segments'
+        self.swift.register('DELETE', segment_bucket + '/object/X',
+                            swob.HTTPNotFound, {}, None)
+        req = Request.blank('/bucket/object?uploadId=X',
+                            environ={'REQUEST_METHOD': 'POST'},
+                            headers={'Authorization': 'AWS test:tester:hmac',
+                                     'Date': self.get_date_header(), },
+                            body=xml)
+        status, headers, body = self.call_swift3(req)
+        self.assertEqual(status.split()[0], '200')
+        fromstring(body, 'CompleteMultipartUploadResult')
+
+        _, _, headers = self.swift.calls_with_headers[-2]
+        self.assertEqual(headers.get('X-Object-Meta-Foo'), 'bar')
+        self.assertEqual(headers.get('Content-Type'), 'baz/quux')
+
+    def test_object_multipart_upload_complete_old_content_type(self):
+        self.swift.register_unconditionally(
+            'HEAD', '/v1/AUTH_test/bucket+segments/object/X',
+            swob.HTTPOk, {"Content-Type": "thingy/dingy"}, None)
+
+        req = Request.blank('/bucket/object?uploadId=X',
+                            environ={'REQUEST_METHOD': 'POST'},
+                            headers={'Authorization': 'AWS test:tester:hmac',
+                                     'Date': self.get_date_header(), },
+                            body=xml)
+        status, headers, body = self.call_swift3(req)
+        fromstring(body, 'CompleteMultipartUploadResult')
+        self.assertEqual(status.split()[0], '200')
+
+        _, _, headers = self.swift.calls_with_headers[-2]
+        self.assertEqual(headers.get('Content-Type'), 'thingy/dingy')
+
+    def test_object_multipart_upload_complete_no_content_type(self):
+        self.swift.register_unconditionally(
+            'HEAD', '/v1/AUTH_test/bucket+segments/object/X',
+            swob.HTTPOk, {"X-Object-Sysmeta-Swift3-Has-Content-Type": "no"},
+            None)
+
+        req = Request.blank('/bucket/object?uploadId=X',
+                            environ={'REQUEST_METHOD': 'POST'},
+                            headers={'Authorization': 'AWS test:tester:hmac',
+                                     'Date': self.get_date_header(), },
+                            body=xml)
+        status, headers, body = self.call_swift3(req)
+        fromstring(body, 'CompleteMultipartUploadResult')
+        self.assertEqual(status.split()[0], '200')
+
+        _, _, headers = self.swift.calls_with_headers[-2]
+        self.assertNotIn('Content-Type', headers)
+
     def test_object_multipart_upload_complete_weird_host_name(self):
         # This happens via boto signature v4
         req = Request.blank('/bucket/object?uploadId=X',
@@ -650,29 +726,38 @@ class TestSwift3MultiUpload(Swift3TestCase):
         self.assertEqual(headers.get('X-Object-Meta-Foo'), 'bar')
 
     def test_object_multipart_upload_complete_segment_too_small(self):
-        msgs = [
-            # pre-2.6.0 swift
-            'Each segment, except the last, must be at least 1234 bytes.',
-            # swift 2.6.0
-            'Index 0: too small; each segment, except the last, must be '
-            'at least 1234 bytes.',
-            # swift 2.7.0+
-            'Index 0: too small; each segment must be at least 1 byte.',
-        ]
+        msg = 'Index 0: too small; each segment must be at least 1 byte.'
 
-        for msg in msgs:
-            req = Request.blank(
-                '/bucket/object?uploadId=X',
-                environ={'REQUEST_METHOD': 'POST'},
-                headers={'Authorization': 'AWS test:tester:hmac',
-                         'Date': self.get_date_header(), },
-                body=xml)
+        req = Request.blank(
+            '/bucket/object?uploadId=X',
+            environ={'REQUEST_METHOD': 'POST'},
+            headers={'Authorization': 'AWS test:tester:hmac',
+                     'Date': self.get_date_header(), },
+            body=xml)
 
-            self.swift.register('PUT', '/v1/AUTH_test/bucket/object',
-                                swob.HTTPBadRequest, {}, msg)
-            status, headers, body = self.call_swift3(req)
-            self.assertEqual(status.split()[0], '400')
-            self.assertEqual(self._get_error_code(body), 'EntityTooSmall')
+        self.swift.register('PUT', '/v1/AUTH_test/bucket/object',
+                            swob.HTTPBadRequest, {}, msg)
+        status, headers, body = self.call_swift3(req)
+        self.assertEqual(status.split()[0], '400')
+        self.assertEqual(self._get_error_code(body), 'EntityTooSmall')
+        self.assertEqual(self._get_error_message(body), msg)
+
+        self.swift.clear_calls()
+        CONF.min_segment_size = 5242880
+        req = Request.blank(
+            '/bucket/object?uploadId=X',
+            environ={'REQUEST_METHOD': 'POST'},
+            headers={'Authorization': 'AWS test:tester:hmac',
+                     'Date': self.get_date_header(), },
+            body=xml)
+
+        status, headers, body = self.call_swift3(req)
+        self.assertEqual(status.split()[0], '400')
+        self.assertEqual(self._get_error_code(body), 'EntityTooSmall')
+        self.assertEqual(self._get_error_message(body),
+                         'Your proposed upload is smaller than the minimum '
+                         'allowed object size.')
+        self.assertNotIn('PUT', [method for method, _ in self.swift.calls])
 
     def test_object_multipart_upload_complete_single_zero_length_segment(self):
         segment_bucket = '/v1/AUTH_test/empty-bucket+segments'
@@ -779,6 +864,69 @@ class TestSwift3MultiUpload(Swift3TestCase):
                     'format=json&prefix=object/X/'),
         ])
 
+    def test_object_multipart_upload_complete_zero_length_final_segment(self):
+        segment_bucket = '/v1/AUTH_test/bucket+segments'
+
+        object_list = [{
+            'name': 'object/X/1',
+            'last_modified': self.last_modified,
+            'hash': 'some hash',
+            'bytes': '100',
+        }, {
+            'name': 'object/X/2',
+            'last_modified': self.last_modified,
+            'hash': 'some other hash',
+            'bytes': '1',
+        }, {
+            'name': 'object/X/3',
+            'last_modified': self.last_modified,
+            'hash': 'd41d8cd98f00b204e9800998ecf8427e',
+            'bytes': '0',
+        }]
+
+        self.swift.register('GET', segment_bucket, swob.HTTPOk, {},
+                            json.dumps(object_list))
+        self.swift.register('HEAD', '/v1/AUTH_test/bucket',
+                            swob.HTTPNoContent, {}, None)
+        self.swift.register('HEAD', segment_bucket + '/object/X',
+                            swob.HTTPOk, {'x-object-meta-foo': 'bar',
+                                          'content-type': 'baz/quux'}, None)
+        self.swift.register('DELETE', segment_bucket + '/object/X/3',
+                            swob.HTTPNoContent, {}, None)
+
+        xml = '<CompleteMultipartUpload>' \
+            '<Part>' \
+            '<PartNumber>1</PartNumber>' \
+            '<ETag>some hash</ETag>' \
+            '</Part>' \
+            '<Part>' \
+            '<PartNumber>2</PartNumber>' \
+            '<ETag>some other hash</ETag>' \
+            '</Part>' \
+            '<Part>' \
+            '<PartNumber>3</PartNumber>' \
+            '<ETag>d41d8cd98f00b204e9800998ecf8427e</ETag>' \
+            '</Part>' \
+            '</CompleteMultipartUpload>'
+
+        req = Request.blank('/bucket/object?uploadId=X',
+                            environ={'REQUEST_METHOD': 'POST'},
+                            headers={'Authorization': 'AWS test:tester:hmac',
+                                     'Date': self.get_date_header(), },
+                            body=xml)
+        status, headers, body = self.call_swift3(req)
+        self.assertEqual(status.split()[0], '200')
+
+        self.assertEqual(self.swift.calls, [
+            ('HEAD', '/v1/AUTH_test/bucket'),
+            ('HEAD', '/v1/AUTH_test/bucket+segments/object/X'),
+            ('GET', '/v1/AUTH_test/bucket+segments?delimiter=/&'
+                    'format=json&prefix=object/X/'),
+            ('PUT', '/v1/AUTH_test/bucket/object?multipart-manifest=put'),
+            ('DELETE', '/v1/AUTH_test/bucket+segments/object/X/3'),
+            ('DELETE', '/v1/AUTH_test/bucket+segments/object/X'),
+        ])
+
     @s3acl(s3acl_only=True)
     def test_object_multipart_upload_complete_s3acl(self):
         acl_headers = encode_acl('object', ACLPublicRead(Owner('test:tester',
@@ -1524,6 +1672,39 @@ class TestSwift3MultiUpload(Swift3TestCase):
         self.assertEqual('bytes=0-9', put_headers['Range'])
         self.assertEqual('/src_bucket/src_obj', put_headers['X-Copy-From'])
 
+    def _test_no_body(self, use_content_length=False,
+                      use_transfer_encoding=False, string_to_md5=''):
+        content_md5 = md5(string_to_md5).digest().encode('base64').strip()
+        with UnreadableInput(self) as fake_input:
+            req = Request.blank(
+                '/bucket/object?uploadId=X',
+                environ={
+                    'REQUEST_METHOD': 'POST',
+                    'wsgi.input': fake_input},
+                headers={
+                    'Authorization': 'AWS test:tester:hmac',
+                    'Date': self.get_date_header(),
+                    'Content-MD5': content_md5},
+                body='')
+            if not use_content_length:
+                req.environ.pop('CONTENT_LENGTH')
+            if use_transfer_encoding:
+                req.environ['HTTP_TRANSFER_ENCODING'] = 'chunked'
+            status, headers, body = self.call_swift3(req)
+        self.assertEqual(status, '400 Bad Request')
+        self.assertEqual(self._get_error_code(body), 'InvalidRequest')
+        self.assertEqual(self._get_error_message(body),
+                         'You must specify at least one part')
+
+    @s3acl
+    def test_object_multi_upload_empty_body(self):
+        self._test_no_body()
+        self._test_no_body(string_to_md5='test')
+        self._test_no_body(use_content_length=True)
+        self._test_no_body(use_content_length=True, string_to_md5='test')
+        self._test_no_body(use_transfer_encoding=True)
+        self._test_no_body(use_transfer_encoding=True, string_to_md5='test')
+
 
 class TestSwift3MultiUploadNonUTC(TestSwift3MultiUpload):
     def setUp(self):
