@@ -27,7 +27,7 @@ from swift3.cfg import CONF
 from swift3.request import Request as S3_Request
 from swift3.request import S3AclRequest, SigV4Request, SIGV4_X_AMZ_DATE_FORMAT
 from swift3.response import InvalidArgument, NoSuchBucket, InternalError, \
-    AccessDenied, SignatureDoesNotMatch
+    AccessDenied, SignatureDoesNotMatch, RequestTimeTooSkewed
 
 
 Fake_ACL_MAP = {
@@ -221,7 +221,7 @@ class TestRequest(Swift3TestCase):
         self.assertEqual(
             result.exception.headers['content-type'], 'application/xml')
 
-    def test_authenticate_delete_Authorization_from_s3req_headers(self):
+    def test_authenticate_delete_Authorization_from_s3req(self):
         req = Request.blank('/bucket/obj',
                             environ={'REQUEST_METHOD': 'GET'},
                             headers={'Authorization': 'AWS test:tester:hmac',
@@ -232,11 +232,12 @@ class TestRequest(Swift3TestCase):
 
             m_swift_resp.return_value = FakeSwiftResponse()
             s3_req = S3AclRequest(req.environ, MagicMock())
-            self.assertTrue('HTTP_AUTHORIZATION' not in s3_req.environ)
-            self.assertTrue('Authorization' not in s3_req.headers)
+            self.assertNotIn('swift3.auth_details', s3_req.environ)
+            self.assertNotIn('HTTP_AUTHORIZATION', s3_req.environ)
+            self.assertNotIn('Authorization', s3_req.headers)
             self.assertEqual(s3_req.token, 'token')
 
-    def test_to_swift_req_Authorization_not_exist_in_swreq_headers(self):
+    def test_to_swift_req_Authorization_not_exist_in_swreq(self):
         container = 'bucket'
         obj = 'obj'
         method = 'GET'
@@ -251,6 +252,7 @@ class TestRequest(Swift3TestCase):
             m_swift_resp.return_value = FakeSwiftResponse()
             s3_req = S3AclRequest(req.environ, MagicMock())
             sw_req = s3_req.to_swift_req(method, container, obj)
+            self.assertNotIn('swift3.auth_details', sw_req.environ)
             self.assertNotIn('HTTP_AUTHORIZATION', sw_req.environ)
             self.assertNotIn('Authorization', sw_req.headers)
             self.assertEqual(sw_req.headers['X-Auth-Token'], 'token')
@@ -389,8 +391,8 @@ class TestRequest(Swift3TestCase):
             'Authorization':
                 'AWS4-HMAC-SHA256 '
                 'Credential=test/20130524/US/s3/aws4_request, '
-                'SignedHeaders=host;%s,'
-                'Signature=X' % included_header,
+                'SignedHeaders=%s,'
+                'Signature=X' % ';'.join(sorted(['host', included_header])),
             'X-Amz-Content-SHA256': '0123456789'}
 
         headers.update(date_header)
@@ -439,6 +441,24 @@ class TestRequest(Swift3TestCase):
             self._test_request_timestamp_sigv4(date_header)
 
         self.assertEqual('403 Forbidden', cm.exception.message)
+        self.assertIn(access_denied_message, cm.exception.body)
+
+        # far-past Date header
+        date_header = {'Date': 'Tue, 07 Jul 999 21:53:04 GMT'}
+        with self.assertRaises(AccessDenied) as cm:
+            self._test_request_timestamp_sigv4(date_header)
+
+        self.assertEqual('403 Forbidden', cm.exception.message)
+        self.assertIn(access_denied_message, cm.exception.body)
+
+        # far-future Date header
+        date_header = {'Date': 'Tue, 07 Jul 9999 21:53:04 GMT'}
+        with self.assertRaises(RequestTimeTooSkewed) as cm:
+            self._test_request_timestamp_sigv4(date_header)
+
+        self.assertEqual('403 Forbidden', cm.exception.message)
+        self.assertIn('The difference between the request time and the '
+                      'current time is too large.', cm.exception.body)
 
     def _test_request_timestamp_sigv2(self, date_header):
         # signature v4 here
@@ -486,6 +506,31 @@ class TestRequest(Swift3TestCase):
         self.assertEqual('403 Forbidden', cm.exception.message)
         self.assertIn(access_denied_message, cm.exception.body)
 
+        # Negative timestamp
+        date_header = {'X-Amz-Date': '00160523T054055Z'}
+        with self.assertRaises(AccessDenied) as cm:
+            self._test_request_timestamp_sigv2(date_header)
+
+        self.assertEqual('403 Forbidden', cm.exception.message)
+        self.assertIn(access_denied_message, cm.exception.body)
+
+        # far-past Date header
+        date_header = {'Date': 'Tue, 07 Jul 999 21:53:04 GMT'}
+        with self.assertRaises(AccessDenied) as cm:
+            self._test_request_timestamp_sigv2(date_header)
+
+        self.assertEqual('403 Forbidden', cm.exception.message)
+        self.assertIn(access_denied_message, cm.exception.body)
+
+        # far-future Date header
+        date_header = {'Date': 'Tue, 07 Jul 9999 21:53:04 GMT'}
+        with self.assertRaises(RequestTimeTooSkewed) as cm:
+            self._test_request_timestamp_sigv2(date_header)
+
+        self.assertEqual('403 Forbidden', cm.exception.message)
+        self.assertIn('The difference between the request time and the '
+                      'current time is too large.', cm.exception.body)
+
     def test_headers_to_sign_sigv4(self):
         environ = {
             'REQUEST_METHOD': 'GET'}
@@ -506,11 +551,10 @@ class TestRequest(Swift3TestCase):
         sigv4_req = SigV4Request(req.environ)
 
         headers_to_sign = sigv4_req._headers_to_sign()
-        self.assertEqual(['host', 'x-amz-content-sha256', 'x-amz-date'],
-                         sorted(headers_to_sign.keys()))
-        self.assertEqual(headers_to_sign['host'], 'localhost:80')
-        self.assertEqual(headers_to_sign['x-amz-date'], x_amz_date)
-        self.assertEqual(headers_to_sign['x-amz-content-sha256'], '0123456789')
+        self.assertEqual(headers_to_sign, [
+            ('host', 'localhost:80'),
+            ('x-amz-content-sha256', '0123456789'),
+            ('x-amz-date', x_amz_date)])
 
         # no x-amz-date
         headers = {
@@ -526,10 +570,9 @@ class TestRequest(Swift3TestCase):
         sigv4_req = SigV4Request(req.environ)
 
         headers_to_sign = sigv4_req._headers_to_sign()
-        self.assertEqual(['host', 'x-amz-content-sha256'],
-                         sorted(headers_to_sign.keys()))
-        self.assertEqual(headers_to_sign['host'], 'localhost:80')
-        self.assertEqual(headers_to_sign['x-amz-content-sha256'], '0123456789')
+        self.assertEqual(headers_to_sign, [
+            ('host', 'localhost:80'),
+            ('x-amz-content-sha256', '0123456789')])
 
         # SignedHeaders says, host and x-amz-date included but there is not
         # X-Amz-Date header
@@ -645,5 +688,69 @@ class TestRequest(Swift3TestCase):
             self.assertEqual(uri, '/bucket/obj1')
             self.assertEqual(req.environ['PATH_INFO'], '/bucket/obj1')
 
+    @patch.object(CONF, 'storage_domain', 's3.amazonaws.com')
+    @patch.object(S3_Request, '_validate_headers', lambda *a: None)
+    def test_check_signature_sigv2(self):
+        # See https://web.archive.org/web/20151226025049/http://
+        # docs.aws.amazon.com//AmazonS3/latest/dev/RESTAuthentication.html
+        req = Request.blank('/photos/puppy.jpg', headers={
+            'Host': 'johnsmith.s3.amazonaws.com',
+            'Date': 'Tue, 27 Mar 2007 19:36:42 +0000',
+            'Authorization': ('AWS AKIAIOSFODNN7EXAMPLE:'
+                              'bWq2s1WEIj+Ydj0vQ697zp+IXMU='),
+        })
+        sigv2_req = S3_Request(req.environ)
+        expected_sts = '\n'.join([
+            'GET',
+            '',
+            '',
+            'Tue, 27 Mar 2007 19:36:42 +0000',
+            '/johnsmith/photos/puppy.jpg',
+        ])
+        self.assertEqual(expected_sts, sigv2_req._string_to_sign())
+        self.assertTrue(sigv2_req.check_signature(
+            'wJalrXUtnFEMI/K7MDENG/bPxRfiCYEXAMPLEKEY'))
+
+        req = Request.blank('/photos/puppy.jpg', method='PUT', headers={
+            'Content-Type': 'image/jpeg',
+            'Content-Length': '94328',
+            'Host': 'johnsmith.s3.amazonaws.com',
+            'Date': 'Tue, 27 Mar 2007 21:15:45 +0000',
+            'Authorization': ('AWS AKIAIOSFODNN7EXAMPLE:'
+                              'MyyxeRY7whkBe+bq8fHCL/2kKUg='),
+        })
+        sigv2_req = S3_Request(req.environ)
+        expected_sts = '\n'.join([
+            'PUT',
+            '',
+            'image/jpeg',
+            'Tue, 27 Mar 2007 21:15:45 +0000',
+            '/johnsmith/photos/puppy.jpg',
+        ])
+        self.assertEqual(expected_sts, sigv2_req._string_to_sign())
+        self.assertTrue(sigv2_req.check_signature(
+            'wJalrXUtnFEMI/K7MDENG/bPxRfiCYEXAMPLEKEY'))
+
+        req = Request.blank(
+            '/?prefix=photos&max-keys=50&marker=puppy',
+            headers={
+                'User-Agent': 'Mozilla/5.0',
+                'Host': 'johnsmith.s3.amazonaws.com',
+                'Date': 'Tue, 27 Mar 2007 19:42:41 +0000',
+                'Authorization': ('AWS AKIAIOSFODNN7EXAMPLE:'
+                                  'htDYFYduRNen8P9ZfE/s9SuKy0U='),
+            })
+        sigv2_req = S3_Request(req.environ)
+        expected_sts = '\n'.join([
+            'GET',
+            '',
+            '',
+            'Tue, 27 Mar 2007 19:42:41 +0000',
+            '/johnsmith/',
+        ])
+        self.assertEqual(expected_sts, sigv2_req._string_to_sign())
+        self.assertTrue(sigv2_req.check_signature(
+            'wJalrXUtnFEMI/K7MDENG/bPxRfiCYEXAMPLEKEY'))
+
 if __name__ == '__main__':
     unittest.main()
